@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2012-2021 Markus Gans                                      *
+* Copyright 2012-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -21,14 +21,17 @@
 ***********************************************************************/
 
 #include <memory>
+#include <utility>
 
 #include "final/dialog/fdialog.h"
 #include "final/fapplication.h"
 #include "final/fevent.h"
+#include "final/ftypes.h"
 #include "final/fwidgetcolors.h"
 #include "final/input/fkeyboard.h"
 #include "final/menu/fmenubar.h"
 #include "final/menu/fmenuitem.h"
+#include "final/util/flog.h"
 #include "final/widget/fcombobox.h"
 #include "final/widget/fstatusbar.h"
 #include "final/widget/ftooltip.h"
@@ -62,24 +65,29 @@ FDialog::~FDialog()  // destructor
   delete dialog_menu;
   dgl_menuitem = nullptr;
 
+  if ( isModal() )
+    unsetModal();
+
   if ( ! FApplication::isQuit() )
     switchToPrevWindow(this);
 
+  auto has_entry = bool( getDialogList() && ! getDialogList()->empty() );
   delDialog(this);
+  auto fapp = FApplication::getApplicationObject();
 
-  if ( isModal() )
-    unsetModal();
+  if ( fapp && has_entry && noVisibleDialog() )
+    fapp->emitCallback("last-dialog-closed");
 }
 
 
 // public methods of FDialog
 //----------------------------------------------------------------------
-bool FDialog::setDialogWidget (bool enable)
+auto FDialog::setDialogWidget (bool enable) -> bool
 {
   if ( isDialogWidget() == enable )
     return true;
 
-  setFlags().dialog_widget = enable;
+  setFlags().type.dialog_widget = enable;
 
   if ( enable )
     setTermOffsetWithPadding();
@@ -90,18 +98,18 @@ bool FDialog::setDialogWidget (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::setModal (bool enable)
+auto FDialog::setModal (bool enable) -> bool
 {
   if ( isModal() == enable )
     return true;
 
-  setFlags().modal = enable;
+  setFlags().visibility.modal = enable;
 
   if ( enable )
   {
     setModalDialogCounter()++;
     FKeyboard::getInstance().clearKeyBuffer();
-    setFlags().minimizable = false;
+    setFlags().feature.minimizable = false;
   }
   else
     setModalDialogCounter()--;
@@ -110,7 +118,7 @@ bool FDialog::setModal (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::setBorder (bool enable)
+auto FDialog::setBorder (bool enable) -> bool
 {
   if ( enable )
   {
@@ -127,7 +135,7 @@ bool FDialog::setBorder (bool enable)
     setRightPadding(0);
   }
 
-  return ( setFlags().no_border = (! enable) );
+  return ( setFlags().feature.no_border = (! enable) );
 }
 
 //----------------------------------------------------------------------
@@ -140,7 +148,7 @@ void FDialog::resetColors()
 }
 
 //----------------------------------------------------------------------
-bool FDialog::setResizeable (bool enable)
+auto FDialog::setResizeable (bool enable) -> bool
 {
   FWindow::setResizeable (enable);
 
@@ -153,7 +161,7 @@ bool FDialog::setResizeable (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::setMinimizable (bool enable)
+auto FDialog::setMinimizable (bool enable) -> bool
 {
   FWindow::setMinimizable (enable);
 
@@ -187,16 +195,17 @@ void FDialog::show()
 void FDialog::hide()
 {
   FWindow::hide();
+  auto fapp = FApplication::getApplicationObject();
+
+  if ( noVisibleDialog() )
+     fapp->emitCallback("last-dialog-closed");
 
   if ( isModal() )
-  {
-    auto fapp = FApplication::getApplicationObject();
     fapp->exitLoop();
-  }
 }
 
 //----------------------------------------------------------------------
-FDialog::ResultCode FDialog::exec()
+auto FDialog::exec() -> ResultCode
 {
   result_code = ResultCode::Reject;
   show();
@@ -213,73 +222,26 @@ void FDialog::setPos (const FPoint& pos, bool)
   if ( isOutsideTerminal(pos) || isZoomed() || getPos() == pos )
   {
     setPos_error = true;
+    new_pos.setPoint (getPos());
     return;
   }
 
-  const int dx = getX() - pos.getX();
-  const int dy = getY() - pos.getY();
-  const int old_x = getTermX();
-  const int old_y = getTermY();
-  const auto& shadow = getShadow();
-  const std::size_t width = getWidth() + shadow.getWidth();  // width + right shadow
-  const std::size_t height = getHeight() + shadow.getHeight();  // height + bottom shadow
-  const FRect old_geometry {getTermGeometryWithShadow()};
+  // Save current position values
+  const auto delta_pos = getPos() - pos;
+  const auto old_geometry = getTermGeometryWithShadow();
 
-  // move to the new position
-  FWindow::setPos(pos, false);
+  // Move to the new position
+  FWindow::setPos (pos, false);
+  new_pos.setPoint (getPos());
+
+  // Copy dialog to virtual terminal
   putArea (getTermPos(), getVWin());
 
-  // restoring the non-covered terminal areas
-  if ( getTermGeometry().overlap(old_geometry) && ! isMinimized() )
-  {
-    FRect restore{};
-    const auto d_width = std::size_t(std::abs(dx));
-    const auto d_height = std::size_t(std::abs(dy));
+  // Restore background outside the dialog
+  recoverBackgroundAfterMove (delta_pos, old_geometry);
 
-    // dx > 0 : move left
-    // dx = 0 : move vertical
-    // dx < 0 : move right
-    // dy > 0 : move up
-    // dy = 0 : move horizontal
-    // dy < 0 : move down
-
-    if ( dx > 0 )
-    {
-      if ( dy > 0 )
-        restore.setRect ( old_x + int(width) - dx, old_y
-                        , d_width, height - d_height );
-      else
-        restore.setRect ( old_x + int(width) - dx, old_y - dy
-                        , d_width, height - d_height );
-    }
-    else
-    {
-      if ( dy > 0 )
-        restore.setRect (old_x, old_y, d_width, height - d_height);
-      else
-        restore.setRect (old_x, old_y - dy, d_width, height - d_height);
-    }
-
-    restoreVTerm (restore);
-
-    if ( dy > 0 )
-      restore.setRect ( old_x, old_y + int(height) - dy, width, d_height);
-    else
-      restore.setRect ( old_x, old_y, width, d_height);
-
-    restoreVTerm (restore);
-  }
-  else
-  {
-    restoreVTerm (old_geometry);
-  }
-
-  restoreOverlaidWindows();
   FWindow::adjustSize();
   setCursorToFocusWidget();
-
-  if ( updateTerminal() )
-    flush();
 }
 
 //----------------------------------------------------------------------
@@ -289,14 +251,13 @@ void FDialog::move (const FPoint& d_pos)
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::moveUp (int n)
+inline auto FDialog::moveUp (int n) -> bool
 {
   if ( isBottomOutside() )
   {
     const auto y_max = int(getMaxHeight());
     FWindow::setY(y_max, false);
     putArea (getTermPos(), getVWin());
-    restoreOverlaidWindows();
   }
   else
     move ({0, -n});
@@ -305,21 +266,20 @@ inline bool FDialog::moveUp (int n)
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::moveDown (int n)
+inline auto FDialog::moveDown (int n) -> bool
 {
   move ({0, n});
   return ! setPos_error;
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::moveLeft (int n)
+inline auto FDialog::moveLeft (int n) -> bool
 {
   if ( isLeftOutside() )
   {
     const auto x_max = int(getMaxWidth());
     FWindow::setX(x_max, false);
     putArea (getTermPos(), getVWin());
-    restoreOverlaidWindows();
   }
   else
     move ({-n, 0});
@@ -328,7 +288,7 @@ inline bool FDialog::moveLeft (int n)
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::moveRight (int n)
+inline auto FDialog::moveRight (int n) -> bool
 {
   move ({n, 0});
   return ! setPos_error;
@@ -339,22 +299,27 @@ void FDialog::setSize (const FSize& size, bool adjust)
 {
   setSize_error = false;
 
-  if ( getSize() == size || isZoomed() )
+  if ( getSize() == size || size.isEmpty() || isZoomed() )
   {
     setSize_error = true;
+    new_size.setSize (getSize());
     return;
   }
 
+  setTerminalUpdates (FVTerm::TerminalUpdate::Stop);
   const int x = getTermX();
   const int y = getTermY();
   const int dw = int(getWidth()) - int(size.getWidth());
   const int dh = int(getHeight()) - int(size.getHeight());
   const auto& shadow = getShadow();
+  const std::size_t old_width = getWidth() + shadow.getWidth();
+  const std::size_t old_height = getHeight() + shadow.getHeight();
   FWindow::setSize (size, false);
+  new_size.setSize (getSize());
 
-  // get adjust width and height
-  const std::size_t w = getWidth() + shadow.getWidth();
-  const std::size_t h = getHeight() + shadow.getHeight();
+  // get adjust width and height with shadow
+  const std::size_t width = getWidth() + shadow.getWidth();
+  const std::size_t height = getHeight() + shadow.getHeight();
 
   // dw > 0 : scale down width
   // dw = 0 : scale only height
@@ -363,33 +328,30 @@ void FDialog::setSize (const FSize& size, bool adjust)
   // dh = 0 : scale only width
   // dh < 0 : scale up height
 
-  const auto d_width = std::size_t(dw);
-  const auto d_height = std::size_t(dh);
-  setTerminalUpdates (FVTerm::TerminalUpdate::Stop);
-
-  // restoring the non-covered terminal areas
-  if ( dw > 0 )
-    restoreVTerm ({x + int(w), y, d_width, h + d_height});  // restore right
-
-  if ( dh > 0 )
-    restoreVTerm ({x, y + int(h), w + d_width, d_height});  // restore bottom
-
   if ( adjust )    // Adjust the size after restoreVTerm(),
     adjustSize();  // because adjustSize() can also change x and y
 
   redraw();
 
-  // handle overlaid windows
-  restoreOverlaidWindows();
+  // Copy dialog to virtual terminal
+  putArea (getTermPos(), getVWin());
+
+  // restoring the non-covered terminal areas
+  if ( dw > 0 )
+    restoreVTerm ({x + int(width), y, std::size_t(dw), old_height});  // restore right
+
+  if ( dh > 0 )
+    restoreVTerm ({x, y + int(height), old_width, std::size_t(dh)});  // restore bottom
 
   // set the cursor to the focus widget
   setCursorToFocusWidget();
 
-  setTerminalUpdates (FVTerm::TerminalUpdate::Start);
+  // Update the terminal
+  setTerminalUpdates (FVTerm::TerminalUpdate::Continue);
 }
 
 //----------------------------------------------------------------------
-bool FDialog::reduceHeight (int n)
+auto FDialog::reduceHeight (int n) -> bool
 {
   if ( ! isResizeable() )
     return false;
@@ -399,7 +361,7 @@ bool FDialog::reduceHeight (int n)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::expandHeight (int n)
+auto FDialog::expandHeight (int n) -> bool
 {
   if ( ! isResizeable() || getHeight() + std::size_t(getY()) > getMaxHeight() )
     return false;
@@ -409,7 +371,7 @@ bool FDialog::expandHeight (int n)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::reduceWidth (int n)
+auto FDialog::reduceWidth (int n) -> bool
 {
   if ( ! isResizeable() )
     return false;
@@ -419,7 +381,7 @@ bool FDialog::reduceWidth (int n)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::expandWidth (int n)
+auto FDialog::expandWidth (int n) -> bool
 {
   if ( ! isResizeable() || getWidth() + std::size_t(getX()) > getMaxWidth() )
     return false;
@@ -429,7 +391,7 @@ bool FDialog::expandWidth (int n)
 }
 
 //----------------------------------------------------------------------
-bool FDialog::zoomWindow()
+auto FDialog::zoomWindow() -> bool
 {
   bool ret_val = FWindow::zoomWindow();
   setZoomItem();
@@ -437,13 +399,23 @@ bool FDialog::zoomWindow()
 }
 
 //----------------------------------------------------------------------
-bool FDialog::minimizeWindow()
+auto FDialog::minimizeWindow() -> bool
 {
   bool ret_val = FWindow::minimizeWindow();
   setMinimizeItem();
   return ret_val;
 }
 
+//----------------------------------------------------------------------
+void FDialog::flushChanges()
+{
+  if ( getSize() != new_size )
+    setSize (new_size);
+
+  if ( getPos() != new_pos )
+    setPos(new_pos);
+}
+
 //----------------------------------------------------------------------
 void FDialog::activateDialog()
 {
@@ -473,8 +445,7 @@ void FDialog::activateDialog()
       old_focus->redraw();
   }
 
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  drawStatusBarMessage();
 }
 
 //----------------------------------------------------------------------
@@ -496,7 +467,7 @@ void FDialog::onKeyPress (FKeyEvent* ev)
   }
 
   // Dialog move and resize functions
-  if ( getMoveSizeWidget() )
+  if ( getMoveResizeWidget() )
     moveSizeKey(ev);
 
   if ( this == getMainWidget() )
@@ -505,6 +476,7 @@ void FDialog::onKeyPress (FKeyEvent* ev)
   if ( ! ev->isAccepted() && isEscapeKey(key) )
   {
     ev->accept();
+    clearStatusBar();
 
     if ( isModal() )
       done (ResultCode::Reject);
@@ -516,59 +488,24 @@ void FDialog::onKeyPress (FKeyEvent* ev)
 //----------------------------------------------------------------------
 void FDialog::onMouseDown (FMouseEvent* ev)
 {
-  const auto width = int(getWidth());
-  const MouseStates ms = initMouseStates(*ev, false);
+  const auto& ms = initMouseStates(*ev, false);
   deactivateMinimizeButton();
   deactivateZoomButton();
 
   if ( ev->getButton() == MouseButton::Left )
   {
-    // Click on titlebar or window: raise + activate
-    raiseActivateDialog();
-
-    if ( isMouseOverTitlebar(ms) )
-      titlebar_click_pos.setPoint (ev->getTermX(), ev->getTermY());
-    else
-      titlebar_click_pos.setPoint (0, 0);
-
-    // Click on titlebar menu button
-    if ( isMouseOverMenuButton(ms) )
-      openMenu();
-    else
-    {
-      activateZoomButton(ms);
-      activateMinimizeButton(ms);
-    }
-
-    // Click on the lower right resize corner
-    resizeMouseDown(ms);
+    handleLeftMouseDown(ms);
   }
-  else  // ev->getButton() != MouseButton::Left
+  else
   {
-    // Click on titlebar menu button
-    if ( isMouseOverMenuButton(ms) && dialog_menu->isShown() )
-      leaveMenu();  // close menu
-
-    cancelMouseResize();  // Cancel resize
+    handleRightAndMiddleMouseDown(ev->getButton(), ms);
   }
-
-  const auto first = int(getMenuButtonWidth() + 1);
-
-  // Click on titlebar: just activate
-  if ( ev->getButton() == MouseButton::Right
-    && ms.mouse_x >= first && ms.mouse_x <= width && ms.mouse_y == 1 )
-      activateDialog();
-
-  // Click on titlebar: lower + activate
-  if ( ev->getButton() == MouseButton::Middle
-    && ms.mouse_x >= first && ms.mouse_x <= width && ms.mouse_y == 1 )
-      lowerActivateDialog();
 }
 
 //----------------------------------------------------------------------
 void FDialog::onMouseUp (FMouseEvent* ev)
 {
-  const MouseStates ms = initMouseStates(*ev, false);
+  const auto& ms = initMouseStates(*ev, false);
 
   if ( ev->getButton() == MouseButton::Left )
   {
@@ -583,6 +520,7 @@ void FDialog::onMouseUp (FMouseEvent* ev)
       const FPoint deltaPos{ms.termPos - titlebar_click_pos};
       move (deltaPos);
       titlebar_click_pos = ms.termPos;
+      ev->setPos(ev->getPos() - deltaPos);
     }
 
     // Click on titlebar menu button
@@ -613,7 +551,7 @@ void FDialog::onMouseUp (FMouseEvent* ev)
 void FDialog::onMouseMove (FMouseEvent* ev)
 {
   auto mouse_over_menu = isMouseOverMenu(ev->getTermPos());
-  const MouseStates ms = initMouseStates(*ev, mouse_over_menu);
+  const auto& ms = initMouseStates(*ev, mouse_over_menu);
 
   if ( ev->getButton() != MouseButton::Left )
     return;
@@ -621,8 +559,9 @@ void FDialog::onMouseMove (FMouseEvent* ev)
   if ( ! titlebar_click_pos.isOrigin() )
   {
     const FPoint deltaPos{ms.termPos - titlebar_click_pos};
-    move (deltaPos);
+    new_pos.setPoint (new_pos + deltaPos);
     titlebar_click_pos = ms.termPos;
+    ev->setPos(ev->getPos() - deltaPos);
   }
 
   // Mouse event handover to the menu
@@ -637,7 +576,7 @@ void FDialog::onMouseMove (FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FDialog::onMouseDoubleClick (FMouseEvent* ev)
 {
-  const MouseStates ms = initMouseStates(*ev, false);
+  const auto& ms = initMouseStates(*ev, false);
 
   if ( ev->getButton() != MouseButton::Left )
     return;
@@ -659,6 +598,7 @@ void FDialog::onMouseDoubleClick (FMouseEvent* ev)
       window_focus_widget->setFocus();
 
     setClickedWidget(nullptr);
+    clearStatusBar();
 
     if ( isModal() )
       done (ResultCode::Reject);
@@ -716,8 +656,7 @@ void FDialog::onWindowActive (FEvent*)
       focusFirstChild();
   }
 
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  drawStatusBarMessage();
 }
 
 //----------------------------------------------------------------------
@@ -767,6 +706,14 @@ void FDialog::onWindowLowered (FEvent*)
 
 
 // protected methods of FDialog
+//----------------------------------------------------------------------
+void FDialog::adjustSize()
+{
+  FWindow::adjustSize();
+  new_pos.setPoint (getPos());
+  new_size.setSize (getSize());
+}
+
 //----------------------------------------------------------------------
 void FDialog::done (ResultCode result)
 {
@@ -777,7 +724,7 @@ void FDialog::done (ResultCode result)
 //----------------------------------------------------------------------
 void FDialog::draw()
 {
-  if ( tooltip && ! getMoveSizeWidget() )
+  if ( ! getMoveResizeWidget() )
   {
     delete tooltip;
     tooltip = nullptr;
@@ -794,7 +741,7 @@ void FDialog::draw()
   drawTitleBar();
   setCursorPos({2, int(getHeight()) - 1});
 
-  if ( getFlags().shadow )
+  if ( getFlags().shadow.shadow )
     drawDialogShadow();
 
   if ( FVTerm::getFOutput()->isMonochron() )
@@ -804,12 +751,10 @@ void FDialog::draw()
 //----------------------------------------------------------------------
 void FDialog::drawDialogShadow()
 {
-  if ( FVTerm::getFOutput()->isMonochron() && ! getFlags().trans_shadow )
+  if ( FVTerm::getFOutput()->isMonochron() && ! getFlags().shadow.trans_shadow )
     return;
 
-  if ( isMinimized() )
-    clearShadow(this);
-  else
+  if ( ! isMinimized() )
     drawShadow(this);
 }
 
@@ -832,7 +777,7 @@ void FDialog::init()
   ignorePadding();
   setDialogWidget();
   // Initialize geometry values
-  setGeometry (FPoint{1, 1}, FSize{10, 10}, false);
+  FWindow::setGeometry (FPoint{1, 1}, FSize{10, 10}, false);
   setMinimumSize (FSize{15, 4});
   mapKeyFunctions();
   addDialog(this);
@@ -973,8 +918,8 @@ void FDialog::initCloseMenuItem (FMenu* menu)
 }
 
 //----------------------------------------------------------------------
-inline FDialog::MouseStates
-    FDialog::initMouseStates (const FMouseEvent& ev, bool mouse_over_menu) const
+inline auto
+    FDialog::initMouseStates (const FMouseEvent& ev, bool mouse_over_menu) const -> MouseStates
 {
   return {
            ev.getX(),
@@ -990,22 +935,66 @@ inline FDialog::MouseStates
 //----------------------------------------------------------------------
 inline void FDialog::mapKeyFunctions()
 {
-  key_map[FKey::Up]            = [this] () { moveUp(1); };
-  key_map[FKey::Down]          = [this] () { moveDown(1); };
-  key_map[FKey::Left]          = [this] () { moveLeft(1); };
-  key_map[FKey::Right]         = [this] () { moveRight(1); };
-  key_map[FKey::Meta_up]       = [this] () { reduceHeight(1); };
-  key_map[FKey::Shift_up]      = [this] () { reduceHeight(1); };
-  key_map[FKey::Meta_down]     = [this] () { expandHeight(1); };
-  key_map[FKey::Shift_down]    = [this] () { expandHeight(1); };
-  key_map[FKey::Meta_left]     = [this] () { reduceWidth(1); };
-  key_map[FKey::Shift_left]    = [this] () { reduceWidth(1); };
-  key_map[FKey::Meta_right]    = [this] () { expandWidth(1); };
-  key_map[FKey::Shift_right]   = [this] () { expandWidth(1); };
-  key_map[FKey::Return]        = [this] () { acceptMoveSize(); };
-  key_map[FKey::Enter]         = [this] () { acceptMoveSize(); };
-  key_map[FKey::Escape]        = [this] () { cancelMoveSize(); };
-  key_map[FKey::Escape_mintty] = [this] () { cancelMoveSize(); };
+  key_map =
+  {
+    { FKey::Up            , [this] () { moveUp(1); } },
+    { FKey::Down          , [this] () { moveDown(1); } },
+    { FKey::Left          , [this] () { moveLeft(1); } },
+    { FKey::Right         , [this] () { moveRight(1); } },
+    { FKey::Meta_up       , [this] () { reduceHeight(1); } },
+    { FKey::Shift_up      , [this] () { reduceHeight(1); } },
+    { FKey::Meta_down     , [this] () { expandHeight(1); } },
+    { FKey::Shift_down    , [this] () { expandHeight(1); } },
+    { FKey::Meta_left     , [this] () { reduceWidth(1); } },
+    { FKey::Shift_left    , [this] () { reduceWidth(1); } },
+    { FKey::Meta_right    , [this] () { expandWidth(1); } },
+    { FKey::Shift_right   , [this] () { expandWidth(1); } },
+    { FKey::Return        , [this] () { acceptMoveSize(); } },
+    { FKey::Enter         , [this] () { acceptMoveSize(); } },
+    { FKey::Escape        , [this] () { cancelMoveSize(); } },
+    { FKey::Escape_mintty , [this] () { cancelMoveSize(); } }
+  };
+}
+
+//----------------------------------------------------------------------
+inline void FDialog::recoverBackgroundAfterMove ( const FPoint& delta_pos
+                                                , const FRect& old_geometry )
+{
+  // restoring the non-covered terminal areas
+  if ( getTermGeometry().overlap(old_geometry) && ! isMinimized() )
+  {
+    const auto& shadow = getShadow();
+    const auto width = getWidth() + shadow.getWidth();  // width + right shadow
+    const auto height = getHeight() + shadow.getHeight();  // height + bottom shadow
+    const int dx = delta_pos.getX();
+    const int dy = delta_pos.getY();
+    const int old_x = old_geometry.getX();
+    const int old_y = old_geometry.getY();
+    const auto delta_width = std::size_t(std::abs(dx));
+    const auto delta_height = std::size_t(std::abs(dy));
+
+    // dx > 0 : move left
+    // dx = 0 : move vertical
+    // dx < 0 : move right
+    // dy > 0 : move up
+    // dy = 0 : move horizontal
+    // dy < 0 : move down
+    int x1 = ( dx > 0 ) ? old_x + int(width) - dx : old_x;
+    int y1 = ( dy > 0 ) ? old_y : old_y - dy;
+    int x2 = old_x;
+    int y2 = ( dy > 0 ) ? old_y + int(height) - dy : old_y;
+    FRect vertical{x1, y1, delta_width, height - delta_height};
+    FRect horizontal{x2, y2, width, delta_height};
+    // Restore the terminal in the two rectangles
+    restoreVTerm (vertical);
+    restoreVTerm (horizontal);
+  }
+  else
+  {
+    // If the terminal does not overlap or the dialog is minimized,
+    // restore the entire old geometry
+    restoreVTerm (old_geometry);
+  }
 }
 
 //----------------------------------------------------------------------
@@ -1014,7 +1003,7 @@ void FDialog::drawBorder()
   if ( ! hasBorder() )
     return;
 
-  if ( (getMoveSizeWidget() == this || ! resize_click_pos.isOrigin() )
+  if ( (getMoveResizeWidget() == this || ! resize_click_pos.isOrigin() )
     && ! isZoomed() )
   {
     const auto& wc = getColorTheme();
@@ -1023,29 +1012,13 @@ void FDialog::drawBorder()
   else
     setColor();
 
-  if ( FVTerm::getFOutput()->isNewFont() )  // Draw a newfont U-shaped frame
-  {
-    const FRect r{FPoint{1, 1}, getSize()};
+  FRect box{{1, 2}, getSize()};
+  box.scaleBy(0, -1);
 
-    for (auto y = r.getY1() + 1; y < r.getY2(); y++)
-    {
-      print() << FPoint{r.getX1(), y}
-              << UniChar::NF_border_line_left        // border left ⎸
-              << FPoint{r.getX2(), y}
-              << UniChar::NF_rev_border_line_right;  // border right⎹
-    }
-
-    print() << r.getLowerLeftPos()
-            << UniChar::NF_border_corner_lower_left        // ⎣
-            << FString{r.getWidth() - 2, UniChar::NF_border_line_bottom}  // _
-            << UniChar::NF_rev_border_corner_lower_right;  // ⎦
-  }
+  if ( FVTerm::getFOutput()->isNewFont() )
+    finalcut::drawNewFontUShapedBox(this, box);  // Draw a newfont U-shaped frame
   else
-  {
-    FRect box{{1, 2}, getSize()};
-    box.scaleBy(0, -1);
     finalcut::drawBorder(this, box);
-  }
 }
 
 //----------------------------------------------------------------------
@@ -1288,25 +1261,13 @@ void FDialog::drawTextBar()
 }
 
 //----------------------------------------------------------------------
-void FDialog::restoreOverlaidWindows()
+void FDialog::clearStatusBar() const
 {
-  // Restoring overlaid windows
-
-  if ( ! getWindowList() || getWindowList()->empty() )
+  if ( ! getStatusBar() )
     return;
 
-  bool overlaid{false};
-
-  for (auto&& window : *getWindowList())
-  {
-    const auto win = static_cast<FWidget*>(window);
-
-    if ( overlaid )
-      putArea (win->getTermPos(), win->getVWin());
-
-    if ( getVWin() == win->getVWin() )
-      overlaid = true;
-  }
+  getStatusBar()->clearMessage();
+  getStatusBar()->drawMessage();
 }
 
 //----------------------------------------------------------------------
@@ -1338,9 +1299,7 @@ void FDialog::leaveMenu()
     getWindowFocusWidget()->setFocus();
 
   redraw();
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  drawStatusBarMessage();
 }
 
 //----------------------------------------------------------------------
@@ -1381,9 +1340,7 @@ void FDialog::selectFirstMenuItem()
     first_item->setFocus();
 
   dialog_menu->redraw();
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  drawStatusBarMessage();
 }
 
 //----------------------------------------------------------------------
@@ -1400,7 +1357,7 @@ void FDialog::setMinimizeItem()
     minimize_item->setStatusbarMessage ("Minimizes the window");
   }
 
-  if ( getFlags().shadow )
+  if ( getFlags().shadow.shadow )
     drawDialogShadow();
 }
 
@@ -1422,13 +1379,13 @@ void FDialog::setZoomItem()
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FDialog::getMenuButtonWidth() const
+inline auto FDialog::getMenuButtonWidth() const -> std::size_t
 {
   return titlebar_buttons ? 3 : 0;
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FDialog::getZoomButtonWidth() const
+inline auto FDialog::getZoomButtonWidth() const -> std::size_t
 {
   if ( titlebar_buttons && isResizeable() )
     return FVTerm::getFOutput()->isNewFont() ? 2 : 3;
@@ -1437,7 +1394,7 @@ inline std::size_t FDialog::getZoomButtonWidth() const
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FDialog::getMinimizeButtonWidth() const
+inline auto FDialog::getMinimizeButtonWidth() const -> std::size_t
 {
   if ( titlebar_buttons && isMinimizable() )
     return FVTerm::getFOutput()->isNewFont() ? 2 : 3;
@@ -1546,7 +1503,7 @@ void FDialog::pressZoomButton (const MouseStates& ms)
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::isMouseOverMenu (const FPoint& termpos) const
+inline auto FDialog::isMouseOverMenu (const FPoint& termpos) const -> bool
 {
   auto menu_geometry = dialog_menu->getTermGeometry();
 
@@ -1555,13 +1512,13 @@ inline bool FDialog::isMouseOverMenu (const FPoint& termpos) const
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::isMouseOverMenuButton (const MouseStates& ms) const
+inline auto FDialog::isMouseOverMenuButton (const MouseStates& ms) const -> bool
 {
   return ( ms.mouse_x <= int(ms.menu_btn) && ms.mouse_y == 1 );
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::isMouseOverZoomButton (const MouseStates& ms) const
+inline auto FDialog::isMouseOverZoomButton (const MouseStates& ms) const -> bool
 {
   return ( isResizeable()
         && ms.mouse_x > int(getWidth() - ms.zoom_btn)
@@ -1570,7 +1527,7 @@ inline bool FDialog::isMouseOverZoomButton (const MouseStates& ms) const
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::isMouseOverMinimizeButton (const MouseStates& ms)  const
+inline auto FDialog::isMouseOverMinimizeButton (const MouseStates& ms)  const -> bool
 {
   return ( isMinimizable()
         && ms.mouse_x > int(getWidth() - ms.minimize_btn - ms.zoom_btn)
@@ -1579,7 +1536,7 @@ inline bool FDialog::isMouseOverMinimizeButton (const MouseStates& ms)  const
 }
 
 //----------------------------------------------------------------------
-inline bool FDialog::isMouseOverTitlebar (const MouseStates& ms) const
+inline auto FDialog::isMouseOverTitlebar (const MouseStates& ms) const -> bool
 {
   return ( ms.mouse_x > int(ms.menu_btn)
         && ms.mouse_x <= int(getWidth() - ms.minimize_btn - ms.zoom_btn)
@@ -1597,20 +1554,70 @@ inline void FDialog::passEventToSubMenu ( const MouseStates& ms
   const auto& g = ms.termPos;
   const auto& p = dialog_menu->termToWidgetPos(g);
   const auto b = ev.getButton();
-  const auto& _ev = \
+  const auto& new_ev = \
       std::make_shared<FMouseEvent>(Event::MouseMove, p, g, b);
   dialog_menu->mouse_down = true;
   setClickedWidget(dialog_menu);
-  dialog_menu->onMouseMove(_ev.get());
+  dialog_menu->onMouseMove(new_ev.get());
+}
+
+//----------------------------------------------------------------------
+inline void FDialog::handleLeftMouseDown (const MouseStates& ms)
+{
+  // Click on titlebar or window: raise + activate
+  raiseActivateDialog();
+
+  if ( isMouseOverTitlebar(ms) )
+  {
+    titlebar_click_pos.setPoint (ms.termPos);
+    new_pos.setPoint (getPos());
+  }
+  else
+    titlebar_click_pos.setPoint (0, 0);
+
+  // Click on titlebar menu button
+  if ( isMouseOverMenuButton(ms) )
+    openMenu();
+  else
+  {
+    activateZoomButton(ms);
+    activateMinimizeButton(ms);
+  }
+
+  // Click on the lower right resize corner
+  resizeMouseDown(ms);
+}
+
+//----------------------------------------------------------------------
+inline void FDialog::handleRightAndMiddleMouseDown ( const MouseButton& button
+                                                   , const MouseStates& ms )
+{
+  // Click on titlebar menu button
+  if ( isMouseOverMenuButton(ms) && dialog_menu->isShown() )
+    leaveMenu();  // close menu
+
+  cancelMouseResize();  // Cancel resize
+  const auto width = int(getWidth());
+  const auto first = int(getMenuButtonWidth() + 1);
+
+  // Click on titlebar: just activate
+  if ( button == MouseButton::Right
+    && ms.mouse_x >= first && ms.mouse_x <= width && ms.mouse_y == 1 )
+      activateDialog();
+
+  // Click on titlebar: lower + activate
+  if ( button == MouseButton::Middle
+    && ms.mouse_x >= first && ms.mouse_x <= width && ms.mouse_y == 1 )
+      lowerActivateDialog();
 }
 
 //----------------------------------------------------------------------
 inline void FDialog::moveSizeKey (FKeyEvent* ev)
 {
-  const auto& entry = key_map[ev->key()];
+  const auto& iter = key_map.find(ev->key());
 
-  if ( entry )
-    entry();
+  if ( iter != key_map.end() )
+    iter->second();
 
   // Accept for all, so that parent widgets will not receive keystrokes
   ev->accept();
@@ -1636,7 +1643,7 @@ inline void FDialog::lowerActivateDialog()
 }
 
 //----------------------------------------------------------------------
-bool FDialog::isOutsideTerminal (const FPoint& pos) const
+auto FDialog::isOutsideTerminal (const FPoint& pos) const -> bool
 {
   return ( pos.getX() + int(getWidth()) <= 1
         || pos.getX() > int(getMaxWidth())
@@ -1645,19 +1652,19 @@ bool FDialog::isOutsideTerminal (const FPoint& pos) const
 }
 
 //----------------------------------------------------------------------
-bool FDialog::isLeftOutside() const
+auto FDialog::isLeftOutside() const -> bool
 {
   return getX() > int(getMaxWidth());
 }
 
 //----------------------------------------------------------------------
-bool FDialog::isBottomOutside() const
+auto FDialog::isBottomOutside() const -> bool
 {
   return getY() > int(getMaxHeight());
 }
 
 //----------------------------------------------------------------------
-bool FDialog::isLowerRightResizeCorner (const MouseStates& ms) const
+auto FDialog::isLowerRightResizeCorner (const MouseStates& ms) const -> bool
 {
   // 3 characters in the lower right corner  |
   //                                         x
@@ -1668,10 +1675,31 @@ bool FDialog::isLowerRightResizeCorner (const MouseStates& ms) const
             || ms.mouse_x == int(getWidth()) ) && ms.mouse_y == int(getHeight()) ) );
 }
 
+//----------------------------------------------------------------------
+auto FDialog::noVisibleDialog() const -> bool
+{
+  // Is true when there is no visible dialog
+
+  if ( ! getDialogList() || getDialogList()->empty() )
+    return true;
+
+  // The number of iterations should be very low with only
+  // one visible element
+  for (const auto& dgl : *getDialogList())
+  {
+    const auto& win = static_cast<FDialog*>(dgl);
+
+    if ( ! win->isWindowHidden() )
+      return false;
+  }
+
+  return true;
+}
+
 //----------------------------------------------------------------------
 void FDialog::resizeMouseDown (const MouseStates& ms)
 {
-  // Click on the lower right resize corner
+  // Click on the lower right resize corner (mouse button down)
 
   if ( isResizeable() && isLowerRightResizeCorner(ms) )
   {
@@ -1687,7 +1715,7 @@ void FDialog::resizeMouseDown (const MouseStates& ms)
       setSize (size, true);
     }
     else
-      drawBorder();
+      redraw();  // with border color change
   }
   else
     resize_click_pos.setPoint (0, 0);
@@ -1696,52 +1724,54 @@ void FDialog::resizeMouseDown (const MouseStates& ms)
 //----------------------------------------------------------------------
 void FDialog::resizeMouseUpMove (const MouseStates& ms, bool mouse_up)
 {
-  // Resize the dialog
-  if ( isResizeable() && ! resize_click_pos.isOrigin() )
+  // Resize dialog on mouse button up or on mouse movements
+
+  if ( ! isResizeable() || resize_click_pos.isOrigin() )
+    return;
+
+  const auto& r = getRootWidget();
+  resize_click_pos = ms.termPos;
+  const int x2 = resize_click_pos.getX();
+  const int y2 = resize_click_pos.getY();
+  int x2_offset{0};
+  int y2_offset{0};
+
+  if ( r )
   {
-    const auto& r = getRootWidget();
+    x2_offset = r->getLeftPadding();
+    y2_offset = r->getTopPadding();
+  }
+
+  if ( ms.termPos != getTermGeometry().getLowerRightPos() )
+  {
+    int w{};
+    int h{};
+    const FPoint deltaPos{ms.termPos - resize_click_pos};
+
+    if ( x2 - x2_offset <= int(getMaxWidth()) )
+      w = resize_click_pos.getX() + deltaPos.getX() - getTermX() + 1;
+    else
+      w = int(getMaxWidth()) - getTermX() + x2_offset + 1;
+
+    if ( y2 - y2_offset <= int(getMaxHeight()) )
+      h = resize_click_pos.getY() + deltaPos.getY() - getTermY() + 1;
+    else
+      h = int(getMaxHeight()) - getTermY() + y2_offset + 1;
+
+    const FSize size ( ( w >= 0) ? std::size_t(w) : 0
+                     , ( h >= 0) ? std::size_t(h) : 0 );
+    new_size.setSize (size);
     resize_click_pos = ms.termPos;
-    const int x2 = resize_click_pos.getX();
-    const int y2 = resize_click_pos.getY();
-    int x2_offset{0};
-    int y2_offset{0};
+  }
 
-    if ( r )
-    {
-      x2_offset = r->getLeftPadding();
-      y2_offset = r->getTopPadding();
-    }
+  if ( mouse_up )
+  {
+    // Reset the border color
+    resize_click_pos.setPoint (0, 0);
 
-    if ( ms.termPos != getTermGeometry().getLowerRightPos() )
-    {
-      int w{};
-      int h{};
-      const FPoint deltaPos{ms.termPos - resize_click_pos};
-
-      if ( x2 - x2_offset <= int(getMaxWidth()) )
-        w = resize_click_pos.getX() + deltaPos.getX() - getTermX() + 1;
-      else
-        w = int(getMaxWidth()) - getTermX() + x2_offset + 1;
-
-      if ( y2 - y2_offset <= int(getMaxHeight()) )
-        h = resize_click_pos.getY() + deltaPos.getY() - getTermY() + 1;
-      else
-        h = int(getMaxHeight()) - getTermY() + y2_offset + 1;
-
-      const FSize size ( ( w >= 0) ? std::size_t(w) : 0
-                       , ( h >= 0) ? std::size_t(h) : 0 );
-      setSize (size);
-    }
-
-    if ( mouse_up )
-    {
-      // Reset the border color
-      resize_click_pos.setPoint (0, 0);
-
-      // redraw() is required to draw the standard (black) border
-      // and client objects with ignorePadding() option.
-      redraw();
-    }
+    // redraw() is required to draw the standard (black) border
+    // and client objects with ignorePadding() option.
+    redraw();
   }
 }
 
@@ -1761,10 +1791,7 @@ void FDialog::cancelMouseResize()
 inline void FDialog::acceptMoveSize()
 {
   setMoveSizeWidget(nullptr);
-
-  if ( tooltip )
-    delete tooltip;
-
+  delete tooltip;
   tooltip = nullptr;
   redraw();
 }
@@ -1773,10 +1800,7 @@ inline void FDialog::acceptMoveSize()
 inline void FDialog::cancelMoveSize()
 {
   setMoveSizeWidget(nullptr);
-
-  if ( tooltip )
-    delete tooltip;
-
+  delete tooltip;
   tooltip = nullptr;
   setPos (save_geometry.getPos());
 
@@ -1790,14 +1814,26 @@ inline void FDialog::cancelMoveSize()
 void FDialog::addDialog (FWidget* obj)
 {
   // Add the dialog object obj to the dialog list
-  if ( getDialogList() )
-    getDialogList()->push_back(obj);
+
+  if ( ! getDialogList() )
+    return;
+
+  if ( getDialogList()->empty() )
+  {
+    auto fapp = FApplication::getApplicationObject();
+
+    if ( fapp )
+      fapp->emitCallback("first-dialog-opened");
+  }
+
+  getDialogList()->push_back(obj);
 }
 
 //----------------------------------------------------------------------
 void FDialog::delDialog (const FWidget* obj)
 {
   // Delete the dialog object obj from the dialog list
+
   if ( ! getDialogList() || getDialogList()->empty() )
     return;
 
@@ -1895,6 +1931,7 @@ void FDialog::cb_close()
   dialog_menu->hide();
   setClickedWidget(nullptr);
   drawTitleBar();
+  clearStatusBar();
   close();
 }
 
