@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2015-2021 Markus Gans                                      *
+* Copyright 2015-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -27,8 +27,10 @@
   #error "Only <final/final.h> can be included directly."
 #endif
 
+#include "final/fconfig.h"
 #include "final/ftypes.h"
 
+
 // ASCII sequences
 #define ESC      "\033"     // Escape
 #define CSI      ESC "["    // Control sequence introducer (7-bit)
@@ -42,6 +44,8 @@
 namespace finalcut
 {
 
+constexpr char fc_release[] = F_VERSION;
+
 //----------------------------------------------------------------------
 // Global constants and enumerations
 //----------------------------------------------------------------------
@@ -62,6 +66,9 @@ enum class Event
   FocusOut,          // focus out
   ChildFocusIn,      // child focus in
   ChildFocusOut,     // child focus out
+  FailAtChildFocus,  // No further focusable child widgets
+  TerminalFocusIn,   // terminal focus in
+  TerminalFocusOut,  // terminal focus out
   WindowActive,      // activate window
   WindowInactive,    // deactivate window
   WindowRaised,      // raise window
@@ -76,7 +83,7 @@ enum class Event
 };
 
 // Internal character encoding
-enum class Encoding : std::size_t
+enum class Encoding
 {
   UTF8,
   VT100,
@@ -252,52 +259,52 @@ enum class UniChar : wchar_t
  * (2) Only supported in use with newfont
  */
 
-constexpr bool operator < (const UniChar& c1, const wchar_t c2) noexcept
+constexpr auto operator < (const UniChar& c1, const wchar_t c2) noexcept -> bool
 {
   return wchar_t(c1) < c2;
 }
 
-constexpr bool operator > (const UniChar& c1, const wchar_t c2) noexcept
+constexpr auto operator > (const UniChar& c1, const wchar_t c2) noexcept -> bool
 {
   return wchar_t(c1) > c2;
 }
 
-constexpr bool operator < (const wchar_t c1, const UniChar& c2) noexcept
+constexpr auto operator < (const wchar_t c1, const UniChar& c2) noexcept -> bool
 {
   return c1 < wchar_t(c2);
 }
 
-constexpr bool operator > (const wchar_t c1, const UniChar& c2) noexcept
+constexpr auto operator > (const wchar_t c1, const UniChar& c2) noexcept -> bool
 {
   return c1 > wchar_t(c2);
 }
 
-constexpr bool operator <= (const UniChar& c1, const wchar_t c2) noexcept
+constexpr auto operator <= (const UniChar& c1, const wchar_t c2) noexcept -> bool
 {
   return wchar_t(c1) <= c2;
 }
 
-constexpr bool operator >= (const UniChar& c1, const wchar_t c2) noexcept
+constexpr auto operator >= (const UniChar& c1, const wchar_t c2) noexcept -> bool
 {
   return wchar_t(c1) >= c2;
 }
 
-constexpr bool operator <= (const wchar_t c1, const UniChar& c2) noexcept
+constexpr auto operator <= (const wchar_t c1, const UniChar& c2) noexcept -> bool
 {
   return c1 <= wchar_t(c2);
 }
 
-constexpr bool operator >= (const wchar_t c1, const UniChar& c2) noexcept
+constexpr auto operator >= (const wchar_t c1, const UniChar& c2) noexcept -> bool
 {
   return c1 >= wchar_t(c2);
 }
 
-constexpr bool operator == (const wchar_t c1, const UniChar& c2) noexcept
+constexpr auto operator == (const wchar_t c1, const UniChar& c2) noexcept -> bool
 {
   return c1 == wchar_t(c2);
 }
 
-constexpr bool operator == (const UniChar& c1, const wchar_t c2) noexcept
+constexpr auto operator == (const UniChar& c1, const wchar_t c2) noexcept -> bool
 {
   return wchar_t(c1) == c2;
 }
@@ -692,6 +699,8 @@ enum class FKey : uInt32
   Shift_Meta_menu            = 0x01600005,  // shifted M-menu
   Ctrl_Meta_menu             = 0x01600006,  // control-M-menu
   Shift_Ctrl_Meta_menu       = 0x01600007,  // shifted control-M-menu
+  Term_Focus_In              = 0x01900000,  // Terminal focus-in event
+  Term_Focus_Out             = 0x01900001,  // Terminal focus-out event
   Escape_mintty              = 0x0200001b,  // mintty Esc
   X11mouse                   = 0x02000020,  // xterm mouse
   Extended_mouse             = 0x02000021,  // SGR extended mouse
@@ -797,73 +806,73 @@ enum class FKey : uInt32
   Incomplete                 = UINT32_MAX
 };
 
-constexpr FKey operator >> (const FKey& k, const uInt32 n) noexcept
+constexpr auto operator >> (const FKey& k, const uInt32 n) noexcept -> FKey
 {
   return FKey(uInt32(k) >> n);
 }
 
-constexpr FKey operator << (const FKey& k, const uInt32 n) noexcept
+constexpr auto operator << (const FKey& k, const uInt32 n) noexcept -> FKey
 {
   return FKey(uInt32(k) << n);
 }
 
-constexpr bool operator < (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator < (const FKey& k1, const uInt32 k2) noexcept -> bool
 {
   return uInt32(k1) < k2;
 }
 
-constexpr bool operator > (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator > (const FKey& k1, const uInt32 k2) noexcept -> bool
 {
   return uInt32(k1) > k2;
 }
 
-constexpr bool operator <= (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator <= (const FKey& k1, const uInt32 k2) noexcept -> bool
 {
   return uInt32(k1) <= k2;
 }
 
-constexpr bool operator >= (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator >= (const FKey& k1, const uInt32 k2) noexcept -> bool
 {
   return uInt32(k1) >= k2;
 }
 
-constexpr FKey operator | (const FKey& k1, const FKey& k2) noexcept
+constexpr auto operator | (const FKey& k1, const FKey& k2) noexcept -> FKey
 {
   return FKey(uInt32(k1) | uInt32(k2));
 }
 
-constexpr FKey operator & (const FKey& k1, const FKey& k2) noexcept
+constexpr auto operator & (const FKey& k1, const FKey& k2) noexcept -> FKey
 {
   return FKey(uInt32(k1) & uInt32(k2));
 }
 
-constexpr FKey operator + (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator + (const FKey& k1, const uInt32 k2) noexcept -> FKey
 {
   return ( k1 < UINT32_MAX - k2 ) ? FKey(uInt32(k1) + k2) : FKey::None;
 }
 
-constexpr FKey operator + (const FKey& k1, const FKey& k2) noexcept
+constexpr auto operator + (const FKey& k1, const FKey& k2) noexcept -> FKey
 {
   return k1 + uInt32(k2);
 }
 
-constexpr FKey operator - (const FKey& k1, const uInt32 k2) noexcept
+constexpr auto operator - (const FKey& k1, const uInt32 k2) noexcept -> FKey
 {
   return ( k1 > k2 ) ? FKey(uInt32(k1) - k2) : FKey::None;
 }
 
-constexpr FKey operator - (const FKey& k1, const FKey& k2) noexcept
+constexpr auto operator - (const FKey& k1, const FKey& k2) noexcept -> FKey
 {
   return k1 - uInt32(k2);
 }
 
-inline FKey& operator += (FKey& k1, uInt32 k2) noexcept
+constexpr auto operator += (FKey& k1, uInt32 k2) noexcept -> FKey&
 {
   k1 = ( k1 < UINT32_MAX - k2 ) ? FKey(uInt32(k1) + k2) : FKey::None;
   return k1;
 }
 
-inline FKey& operator -= (FKey& k1, uInt32 k2) noexcept
+constexpr auto operator -= (FKey& k1, uInt32 k2) noexcept -> FKey&
 {
   k1 = ( k1 > k2 ) ? FKey(uInt32(k1) - k2) : FKey::None;
   return k1;
@@ -1134,91 +1143,87 @@ enum class FColor : uInt16
   Undefined         = static_cast<uInt16>(-2)
 };
 
-constexpr FColor operator >> (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator >> (const FColor& c, const uInt16 n) noexcept -> FColor
 {
   return FColor(uInt16(c) >> n);
 }
 
-constexpr FColor operator << (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator << (const FColor& c, const uInt16 n) noexcept -> FColor
 {
   return FColor(uInt16(c) << n);
 }
 
-constexpr bool operator < (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator < (const FColor& c, const uInt16 n) noexcept -> bool
 {
   return uInt16(c) < n;
 }
 
-constexpr bool operator > (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator > (const FColor& c, const uInt16 n) noexcept -> bool
 {
   return uInt16(c) > n;
 }
 
-constexpr bool operator == (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator == (const FColor& c, const uInt16 n) noexcept -> bool
 {
   return uInt16(c) == n;
 }
 
-constexpr bool operator <= (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator <= (const FColor& c, const uInt16 n) noexcept -> bool
 {
   return uInt16(c) <= n;
 }
 
-constexpr bool operator >= (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator >= (const FColor& c, const uInt16 n) noexcept -> bool
 {
   return uInt16(c) >= n;
 }
 
-constexpr FColor operator + (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator + (const FColor& c, const uInt16 n) noexcept -> FColor
 {
   return FColor(uInt16(c) + n);
 }
 
-constexpr FColor operator - (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator - (const FColor& c, const uInt16 n) noexcept -> FColor
 {
   return FColor(uInt16(c) - n);
 }
 
-constexpr FColor operator % (const FColor& c, const uInt16 n) noexcept
+constexpr auto operator % (const FColor& c, const uInt16 n) noexcept -> FColor
 {
   return FColor(uInt16(c) % n);
 }
 
-inline FColor& operator %= (FColor& c, uInt16 n) noexcept
+constexpr auto operator %= (FColor& c, uInt16 n) noexcept -> FColor&
 {
-  c =  FColor(uInt16(c) % n);
+  c = FColor(uInt16(c) % n);
   return c;
 }
 
-inline FColor& operator ++ (FColor& c) noexcept  // prefix
+constexpr auto operator ++ (FColor& c) noexcept -> FColor&  // prefix
 {
   c = ( uInt16(c) < 255 ) ? FColor(uInt16(c) + 1) : FColor::Default;
   return c;
 }
 
-inline FColor operator ++ (FColor& c, int) noexcept  // postfix
+constexpr auto operator ++ (FColor& c, int) noexcept -> FColor  // postfix
 {
   const FColor tmp = c;
   ++c;
   return tmp;
 }
 
-inline FColor& operator -- (FColor& c) noexcept  // prefix
+constexpr auto operator -- (FColor& c) noexcept -> FColor&  // prefix
 {
-  c = [&c] ()
-  {
-    if ( uInt16(c) > 0 )
-      return FColor(uInt16(c) - 1);
+  if ( uInt16(c) > 0 )
+    return (c = FColor(uInt16(c) - 1));
 
-    if ( c == FColor::Black )  // value 0
-      return FColor::Default;  // value uInt16(-1)
+  if ( c == FColor::Black )        // value 0
+    return (c = FColor::Default);  // value uInt16(-1)
 
-    return FColor(255);        // --(-1) = 255
-  }();
-  return c;
+  return (c = FColor(255));        // --(-1) = 255
 }
 
-inline FColor operator -- (FColor& c, int) noexcept  // postfix
+constexpr auto operator -- (FColor& c, int) noexcept -> FColor  // postfix
 {
   const FColor tmp = c;
   --c;
@@ -1246,24 +1251,35 @@ enum class Style : uInt16
   InheritBackground = 1U << 13U
 };
 
-constexpr Style operator + (const Style& a1, const Style& a2) noexcept
+constexpr auto operator + (const Style& a1, const Style& a2) noexcept -> Style
 {
   return Style(uInt16(a1) + uInt16(a2));
 }
 
-constexpr Style operator | (const Style& a1, const Style& a2) noexcept
+constexpr auto operator - (const Style& a) noexcept -> Style
+{
+  return Style(-(uInt16(a)));
+}
+
+constexpr auto operator | (const Style& a1, const Style& a2) noexcept -> Style
 {
   return Style(uInt16(a1) | uInt16(a2));
 }
 
-constexpr Style operator & (const Style& a1, const Style& a2) noexcept
+constexpr auto operator & (const Style& a1, const Style& a2) noexcept -> Style
 {
   return Style(uInt16(a1) & uInt16(a2));
 }
 
+constexpr auto operator ^= (Style& a, const Style& b) noexcept -> Style&
+{
+  a = Style(uInt16(a) ^ uInt16(b));
+  return a;
+}
+
 
 // Mouse/keyboard state values
-enum class MouseButton : int
+enum class MouseButton : uInt
 {
   None    = 0x00,
   Left    = 0x01,
@@ -1274,23 +1290,23 @@ enum class MouseButton : int
   Meta    = 0x20
 };
 
-constexpr MouseButton operator | (const MouseButton& b1, const MouseButton& b2) noexcept
+constexpr auto operator | (const MouseButton& b1, const MouseButton& b2) noexcept -> MouseButton
 {
-  return MouseButton(int(b1) | int(b2));
+  return MouseButton(uInt(b1) | uInt(b2));
 }
 
-inline MouseButton& operator |= (MouseButton& b1, const MouseButton& b2) noexcept
+constexpr auto operator |= (MouseButton& b1, const MouseButton& b2) noexcept -> MouseButton&
 {
   b1 = b1 | b2;
   return b1;
 }
 
-constexpr MouseButton operator & (const MouseButton& b1, const MouseButton& b2) noexcept
+constexpr auto operator & (const MouseButton& b1, const MouseButton& b2) noexcept -> MouseButton
 {
-  return MouseButton(int(b1) & int(b2));
+  return MouseButton(uInt(b1) & uInt(b2));
 }
 
-inline MouseButton& operator &= (MouseButton& b1, const MouseButton& b2) noexcept
+constexpr auto operator &= (MouseButton& b1, const MouseButton& b2) noexcept -> MouseButton&
 {
   b1 = b1 & b2;
   return b1;
@@ -1319,31 +1335,32 @@ enum class FTermType : uInt32
   tera_term      = 1U << 8U,   // Tera Term
   cygwin         = 1U << 9U,   // Cygwin
   mintty         = 1U << 10U,  // Mintty
-  linux_con      = 1U << 11U,  // Linux Console
-  freebsd_con    = 1U << 12U,  // FreeBSD workstation Console
-  netbsd_con     = 1U << 13U,  // NetBSD workstation console
-  openbsd_con    = 1U << 14U,  // OpenBSD workstation Console
-  sun_con        = 1U << 15U,  // Sun Microsystems workstation console
-  screen         = 1U << 16U,  // GNU Screen
-  tmux           = 1U << 17U,  // TMux (terminal multiplexer)
-  kterm          = 1U << 18U,  // Kterm (multi-lingual terminal emulator)
-  mlterm         = 1U << 19U,  // MLterm (multi-lingual terminal emulator)
-  kitty          = 1U << 20U   // kitty (GPU based terminal emulator)
+  stterm         = 1U << 11U,  // st - simple terminal
+  linux_con      = 1U << 12U,  // Linux Console
+  freebsd_con    = 1U << 13U,  // FreeBSD workstation Console
+  netbsd_con     = 1U << 14U,  // NetBSD workstation console
+  openbsd_con    = 1U << 15U,  // OpenBSD workstation Console
+  sun_con        = 1U << 16U,  // Sun Microsystems workstation console
+  screen         = 1U << 17U,  // GNU Screen
+  tmux           = 1U << 18U,  // TMux (terminal multiplexer)
+  kterm          = 1U << 19U,  // Kterm (multi-lingual terminal emulator)
+  mlterm         = 1U << 20U,  // MLterm (multi-lingual terminal emulator)
+  kitty          = 1U << 21U   // kitty (GPU based terminal emulator)
 };
 
-using FTermTypeT = typename std::underlying_type<FTermType>::type;
+using FTermTypeT = std::underlying_type_t<FTermType>;
 
-constexpr FTermTypeT operator | (const FTermType& t1, const FTermType& t2) noexcept
+constexpr auto operator | (const FTermType& t1, const FTermType& t2) noexcept -> FTermTypeT
 {
   return FTermTypeT(t1) | FTermTypeT(t2);
 }
 
-constexpr FTermTypeT operator | (FTermTypeT t1, const FTermType& t2) noexcept
+constexpr auto operator | (FTermTypeT t1, const FTermType& t2) noexcept -> FTermTypeT
 {
   return t1 | FTermTypeT(t2);
 }
 
-constexpr FTermTypeT operator | (const FTermType& t1, FTermTypeT t2) noexcept
+constexpr auto operator | (const FTermType& t1, FTermTypeT t2) noexcept -> FTermTypeT
 {
   return FTermTypeT(t1) | t2;
 }
@@ -1354,7 +1371,7 @@ enum class FocusTypes
 {
   NextWidget     = 0x00,
   PreviousWidget = 0x01,
-  DefiniteWidget = 0x03  // event default
+  DefiniteWidget = 0x03  // Focus event default value
 };
 
 // Mouse drag scrolling mode
@@ -1371,6 +1388,11 @@ enum class DragScrollMode
   SelectRightward = 8
 };
 
+constexpr auto isDragging (const DragScrollMode& drag_scroll) noexcept -> bool
+{
+  return drag_scroll != DragScrollMode::None;
+}
+
 // Scroll bar visibility mode
 enum class ScrollBarMode
 {
