@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -68,7 +68,6 @@
 #include <cstddef>
 #include <functional>
 #include <memory>
-#include <queue>
 #include <utility>
 #include <vector>
 
@@ -89,6 +88,7 @@ namespace finalcut
 //----------------------------------------------------------------------
 // class FMouseData
 //----------------------------------------------------------------------
+
 class FMouseData
 {
   public:
@@ -105,32 +105,32 @@ class FMouseData
     virtual ~FMouseData() noexcept;
 
     // copy assignment operator (=)
-    FMouseData& operator = (const FMouseData&) = default;
+    auto operator = (const FMouseData&) -> FMouseData& = default;
 
     // Move assignment operator (=)
-    FMouseData& operator = (FMouseData&&) noexcept = default;
+    auto operator = (FMouseData&&) noexcept -> FMouseData& = default;
 
     // Accessors
-    virtual FString       getClassName() const;
-    const FPoint&         getPos() const & noexcept;
+    virtual auto getClassName() const -> FString;
+    auto getPos() const & noexcept -> const FPoint&;
 
     // Inquiries
-    bool                  isLeftButtonPressed() const noexcept;
-    bool                  isLeftButtonReleased() const noexcept;
-    bool                  isLeftButtonDoubleClick() const noexcept;
-    bool                  isRightButtonPressed() const noexcept;
-    bool                  isRightButtonReleased() const noexcept;
-    bool                  isMiddleButtonPressed() const noexcept;
-    bool                  isMiddleButtonReleased() const noexcept;
-    bool                  isShiftKeyPressed() const noexcept;
-    bool                  isControlKeyPressed() const noexcept;
-    bool                  isMetaKeyPressed() const noexcept;
-    bool                  isWheelUp() const noexcept;
-    bool                  isWheelDown() const noexcept;
-    bool                  isMoved() const noexcept;
+    auto isLeftButtonPressed() const noexcept -> bool;
+    auto isLeftButtonReleased() const noexcept -> bool;
+    auto isLeftButtonDoubleClick() const noexcept -> bool;
+    auto isRightButtonPressed() const noexcept -> bool;
+    auto isRightButtonReleased() const noexcept -> bool;
+    auto isMiddleButtonPressed() const noexcept -> bool;
+    auto isMiddleButtonReleased() const noexcept -> bool;
+    auto isShiftKeyPressed() const noexcept -> bool;
+    auto isControlKeyPressed() const noexcept -> bool;
+    auto isMetaKeyPressed() const noexcept -> bool;
+    auto isWheelUp() const noexcept -> bool;
+    auto isWheelDown() const noexcept -> bool;
+    auto isMoved() const noexcept -> bool;
 
     // Methods
-    void                  clearButtonState() noexcept;
+    void clearButtonState() noexcept;
 
   protected:
     // Enumerations
@@ -156,16 +156,16 @@ class FMouseData
     };
 
     // Accessors
-    FMouseButton&         getButtonState() & noexcept;
-    const FMouseButton&   getButtonState() const & noexcept;
+    auto getButtonState() & noexcept -> FMouseButton&;
+    auto getButtonState() const & noexcept -> const FMouseButton&;
 
     // Mutator
-    void                  setPos (const FPoint&) noexcept;
+    void setPos (const FPoint&) noexcept;
 
   private:
     // Data members
-    FMouseButton          b_state{};
-    FPoint                mouse{0, 0};  // mouse click position
+    FMouseButton b_state{};
+    FPoint       mouse{0, 0};  // mouse click position
 };
 
 
@@ -190,62 +190,63 @@ class FMouse : public FMouseData
     FMouse();
 
     // Accessors
-    FString               getClassName() const override;
-    MouseType             getMouseTypeID() const noexcept;
-    void                  clearEvent() noexcept;
+    auto getClassName() const -> FString override;
+    auto getMouseTypeID() const noexcept -> MouseType;
+    void clearEvent() noexcept;
 
     // Mutators
-    void                  setMaxWidth (uInt16) noexcept;
-    void                  setMaxHeight (uInt16) noexcept;
-    void                  setDblclickInterval (const uInt64) noexcept;
+    void setMaxWidth (uInt16) noexcept;
+    void setMaxHeight (uInt16) noexcept;
+    void setDblclickInterval (const uInt64) noexcept;
 
     // Inquiries
-    virtual bool          hasData() = 0;
-    bool                  hasEvent() const noexcept;
-    bool                  hasUnprocessedInput() const noexcept;
+    virtual auto hasData() -> bool = 0;
+    auto hasEvent() const noexcept -> bool;
+    auto hasUnprocessedInput() const noexcept -> bool;
 
     // Methods
     template <typename ClassT>
-    static auto           createMouseObject() -> std::unique_ptr<ClassT> ;
-    virtual void          setRawData (FKeyboard::keybuffer&) = 0;
-    virtual void          processEvent (const TimeValue&) = 0;
+    static auto  createMouseObject() -> std::unique_ptr<ClassT>;
+    virtual void setRawData (FKeyboard::keybuffer&) = 0;
+    virtual void processEvent (const TimeValue&) = 0;
 
   protected:
     // Accessors
-    const FPoint&         getNewPos() const & noexcept;
-    uInt16                getMaxWidth() const noexcept;
-    uInt16                getMaxHeight() const noexcept;
-    uInt64                getDblclickInterval() const noexcept;
-    TimeValue             getMousePressedTime() const noexcept;
+    auto getNewPos() const & noexcept -> const FPoint&;
+    auto getMaxWidth() const noexcept -> uInt16;
+    auto getMaxHeight() const noexcept -> uInt16;
+    auto getDblclickInterval() const noexcept -> uInt64;
+    auto getMousePressedTime() const noexcept -> TimeValue;
 
     // Mutator
-    void                  setMouseTypeID (MouseType) noexcept;
-    void                  setNewPos (int, int) noexcept;
-    void                  setPending (bool = true) noexcept;
-    void                  setEvent() noexcept;
-    void                  setMousePressedTime (const TimeValue&) noexcept;
-    void                  resetMousePressedTime() noexcept;
+    void setMouseTypeID (MouseType) noexcept;
+    void setNewPos (int, int) noexcept;
+    void useNewPos() noexcept;
+    void setPending (bool = true) noexcept;
+    void setEvent() noexcept;
+    void setMousePressedTime (const TimeValue&) noexcept;
+    void resetMousePressedTime() noexcept;
 
     // Inquiry
-    bool                  isDblclickTimeout (const TimeValue&) const;
+    auto isDblclickTimeout (const TimeValue&) const -> bool;
 
   private:
     // Data members
-    bool                  mouse_event_occurred{false};
-    bool                  unprocessed_buffer_data{false};
-    MouseType             MouseType_id{MouseType::None};
-    uInt16                max_width{80};
-    uInt16                max_height{25};
-    uInt64                dblclick_interval{500000};  // 500 ms
-    TimeValue             time_mousepressed{};
-    FPoint                new_mouse_position{};
+    bool       mouse_event_occurred{false};
+    bool       unprocessed_buffer_data{false};
+    MouseType  MouseType_id{MouseType::None};
+    uInt16     max_width{80};
+    uInt16     max_height{25};
+    uInt64     dblclick_interval{500000};  // 500 ms
+    TimeValue  time_mousepressed{};
+    FPoint     new_mouse_position{};
 };
 
 //----------------------------------------------------------------------
 template <typename ClassT>
 inline auto FMouse::createMouseObject() -> std::unique_ptr<ClassT>
 {
-  return make_unique<ClassT>();
+  return std::make_unique<ClassT>();
 }
 
 
@@ -261,26 +262,26 @@ class FMouseGPM final : public FMouse
     FMouseGPM();
 
     // Accessors
-    FString              getClassName() const override;
+    auto getClassName() const -> FString override;
 
     // Mutators
-    void                 setStdinNo(int) noexcept;
+    void setStdinNo(int) noexcept;
 
     // Inquiry
-    bool                 hasData() noexcept override;
-    bool                 isGpmMouseEnabled() const noexcept;
+    auto hasData() noexcept -> bool override;
+    auto isGpmMouseEnabled() const noexcept -> bool;
 
     // Methods
-    void                 setRawData (FKeyboard::keybuffer&) noexcept override;
-    void                 processEvent (const TimeValue&) override;
-    bool                 gpmMouse (bool = true);
-    bool                 enableGpmMouse();
-    bool                 disableGpmMouse();
-    bool                 hasSignificantEvents() const noexcept;
-    void                 interpretKeyDown() noexcept;
-    void                 interpretKeyUp() noexcept;
-    bool                 getGpmKeyPressed (bool = true);
-    void                 drawPointer() const;
+    void setRawData (FKeyboard::keybuffer&) noexcept override;
+    void processEvent (const TimeValue&) override;
+    auto gpmMouse (bool = true) -> bool;
+    auto enableGpmMouse() -> bool;
+    auto disableGpmMouse() -> bool;
+    auto hasSignificantEvents() const noexcept -> bool;
+    void interpretKeyDown() noexcept;
+    void interpretKeyUp() noexcept;
+    auto getGpmKeyPressed (bool = true) -> bool;
+    void drawPointer() const;
 
   private:
     // Enumeration
@@ -292,25 +293,25 @@ class FMouseGPM final : public FMouse
     };
 
     // Method
-    gpmEventType       gpmEvent (bool = true) const;
+    auto         gpmEvent (bool = true) const -> gpmEventType;
 
     // Data member
-    Gpm_Event          gpm_ev{};
-    bool               has_gpm_mouse_data{false};
-    bool               gpm_mouse_enabled{false};
-    int                stdin_no{0};
+    Gpm_Event gpm_ev{};
+    bool      has_gpm_mouse_data{false};
+    bool      gpm_mouse_enabled{false};
+    int       stdin_no{0};
 };
 
 //----------------------------------------------------------------------
-inline bool FMouseGPM::enableGpmMouse()
+inline auto FMouseGPM::enableGpmMouse() -> bool
 { return gpmMouse(true); }
 
 //----------------------------------------------------------------------
-inline bool FMouseGPM::disableGpmMouse()
+inline auto FMouseGPM::disableGpmMouse() -> bool
 { return gpmMouse(false); }
 
 //----------------------------------------------------------------------
-inline bool FMouseGPM::isGpmMouseEnabled() const noexcept
+inline auto FMouseGPM::isGpmMouseEnabled() const noexcept -> bool
 { return gpm_mouse_enabled; }
 #endif  // F_HAVE_LIBGPM
 
@@ -326,14 +327,14 @@ class FMouseX11 final : public FMouse
     FMouseX11();
 
     // Accessors
-    FString              getClassName() const override;
+    auto getClassName() const -> FString override;
 
     // Inquiry
-    bool                 hasData() noexcept override;
+    auto hasData() noexcept -> bool override;
 
     // Methods
-    void                 setRawData (FKeyboard::keybuffer&) noexcept override;
-    void                 processEvent (const TimeValue&) override;
+    void setRawData (FKeyboard::keybuffer&) noexcept override;
+    void processEvent (const TimeValue&) override;
 
   private:
     // Enumeration
@@ -361,12 +362,14 @@ class FMouseX11 final : public FMouse
     static constexpr std::size_t MOUSE_BUF_SIZE = 4;
 
     // Methods
-    void         setKeyState (int) noexcept;
-    void         setMoveState (const FPoint&, int) noexcept;
-    void         setButtonState (const int, const TimeValue&) noexcept;
+    void setKeyState (int) noexcept;
+    void setMoveState (const FPoint&, int) noexcept;
+    void setButtonState (const int, const TimeValue&) noexcept;
+    void handleButton1Pressed (const TimeValue& time) noexcept;
+    void handleButtonRelease() noexcept;
 
     // Data member
-    char  x11_mouse[MOUSE_BUF_SIZE]{'\0'};
+    std::array<char, MOUSE_BUF_SIZE>x11_mouse{};
     uChar x11_button_state{all_buttons_released};
 };
 
@@ -382,14 +385,14 @@ class FMouseSGR final : public FMouse
     FMouseSGR();
 
     // Accessors
-    FString       getClassName() const override;
+    auto getClassName() const -> FString override;
 
     // Inquiry
-    bool          hasData() noexcept override;
+    auto hasData() noexcept -> bool override;
 
     // Methods
-    void          setRawData (FKeyboard::keybuffer&) noexcept override;
-    void          processEvent (const TimeValue&) override;
+    void setRawData (FKeyboard::keybuffer&) noexcept override;
+    void processEvent (const TimeValue&) override;
 
   private:
     // Enumeration
@@ -416,13 +419,14 @@ class FMouseSGR final : public FMouse
     static constexpr std::size_t MOUSE_BUF_SIZE = 13;
 
     // Methods
-    void          setKeyState (int) noexcept;
-    void          setMoveState (const FPoint&, int) noexcept;
-    void          setPressedButtonState (const int, const TimeValue&) noexcept;
-    void          setReleasedButtonState (const int) noexcept;
+    void setKeyState (int) noexcept;
+    void setMoveState (const FPoint&, int) noexcept;
+    void setPressedButtonState (const int, const TimeValue&) noexcept;
+    void handleButton1Pressed (const TimeValue& time) noexcept;
+    void setReleasedButtonState (const int) noexcept;
 
     // Data members
-    char  sgr_mouse[MOUSE_BUF_SIZE]{'\0'};
+    std::array<char, MOUSE_BUF_SIZE> sgr_mouse{};
     uChar sgr_button_state{0x23};
 };
 
@@ -438,14 +442,14 @@ class FMouseUrxvt final : public FMouse
     FMouseUrxvt();
 
     // Accessors
-    FString       getClassName() const override;
+    auto getClassName() const -> FString override;
 
     // Inquiry
-    bool          hasData() noexcept override;
+    auto hasData() noexcept -> bool override;
 
     // Methods
-    void          setRawData (FKeyboard::keybuffer&) noexcept override;
-    void          processEvent (const TimeValue&) override;
+    void setRawData (FKeyboard::keybuffer&) noexcept override;
+    void processEvent (const TimeValue&) override;
 
   private:
     // Enumeration
@@ -470,15 +474,16 @@ class FMouseUrxvt final : public FMouse
     };
 
     // Constant
-    static constexpr std::size_t MOUSE_BUF_SIZE = 13;
+    static constexpr std::size_t MOUSE_BUF_SIZE = 14;
 
     // Methods
-    void          setKeyState (int) noexcept;
-    void          setMoveState (const FPoint&, int) noexcept;
-    void          setButtonState (const int, const TimeValue&) noexcept;
+    void setKeyState (int) noexcept;
+    void setMoveState (const FPoint&, int) noexcept;
+    void setButtonState (const int, const TimeValue&) noexcept;
+    void handleButtonRelease() noexcept;
 
     // Data members
-    char  urxvt_mouse[MOUSE_BUF_SIZE]{'\0'};
+    std::array<char, MOUSE_BUF_SIZE>  urxvt_mouse{};
     uChar urxvt_button_state{all_buttons_released};
 };
 
@@ -498,7 +503,7 @@ class FMouseCommand final
 
     // Method
     template <typename T>
-    void execute(T&& arg) const
+    inline void execute(T&& arg) const
     {
       handler(std::forward<T>(arg));
     }
@@ -516,6 +521,9 @@ class FMouseCommand final
 class FMouseControl
 {
   public:
+    // Using-declaration
+    using FMouseDataPtr = std::shared_ptr<FMouseData>;
+
     // Constructor
     FMouseControl();
 
@@ -523,91 +531,93 @@ class FMouseControl
     virtual ~FMouseControl();
 
     // Accessors
-    virtual FString           getClassName() const;
-    static auto               getInstance() -> FMouseControl&;
-    const FPoint&             getPos() &;
-    FMouseData*               getCurrentMouseEvent() const;
-    void                      clearEvent();
+    virtual auto getClassName() const -> FString;
+    static auto  getInstance() -> FMouseControl&;
+    static auto  getCurrentMouseEvent() -> FMouseDataPtr&;
+    auto  getPos() & -> const FPoint&;
+    void  clearEvent();
 
     // Mutators
-    void                      setStdinNo (int);
-    void                      setMaxWidth (uInt16);
-    void                      setMaxHeight (uInt16);
-    void                      setDblclickInterval (const uInt64) const;
-    void                      setEventCommand (const FMouseCommand&);
-    void                      useGpmMouse (bool = true);
-    void                      useXtermMouse (bool = true);
+    void  setStdinNo (int);
+    void  setMaxWidth (uInt16);
+    void  setMaxHeight (uInt16);
+    void  setDblclickInterval (const uInt64) const;
+    void  setEventCommand (const FMouseCommand&);
+    void  useGpmMouse (bool = true);
+    void  useXtermMouse (bool = true);
 
     // Inquiries
-    bool                      hasData();
-    bool                      hasEvent();
-    bool                      isLeftButtonPressed();
-    bool                      isLeftButtonReleased();
-    bool                      isLeftButtonDoubleClick();
-    bool                      isRightButtonPressed();
-    bool                      isRightButtonReleased();
-    bool                      isMiddleButtonPressed();
-    bool                      isMiddleButtonReleased();
-    bool                      isShiftKeyPressed();
-    bool                      isControlKeyPressed();
-    bool                      isMetaKeyPressed();
-    bool                      isWheelUp();
-    bool                      isWheelDown();
-    bool                      isMoved();
-    bool                      hasUnprocessedInput();
-    bool                      hasDataInQueue() const;
-    bool                      isGpmMouseEnabled() noexcept;
+    auto  hasData() -> bool;
+    auto  hasEvent() -> bool;
+    auto  isLeftButtonPressed() -> bool;
+    auto  isLeftButtonReleased() -> bool;
+    auto  isLeftButtonDoubleClick() -> bool;
+    auto  isRightButtonPressed() -> bool;
+    auto  isRightButtonReleased() -> bool;
+    auto  isMiddleButtonPressed() -> bool;
+    auto  isMiddleButtonReleased() -> bool;
+    auto  isShiftKeyPressed() -> bool;
+    auto  isControlKeyPressed() -> bool;
+    auto  isMetaKeyPressed() -> bool;
+    auto  isWheelUp() -> bool;
+    auto  isWheelDown() -> bool;
+    auto  isMoved() -> bool;
+    auto  hasUnprocessedInput() const -> bool;
+    auto  hasDataInQueue() const -> bool;
+    auto  isGpmMouseEnabled() noexcept -> bool;
 
     // Methods
-    void                      enable();
-    void                      disable();
-    virtual void              setRawData ( const FMouse::MouseType&
-                                         , FKeyboard::keybuffer& );
-    virtual void              processEvent (const TimeValue&);
-    void                      processQueuedInput();
-    bool                      getGpmKeyPressed (bool = true);
-    void                      drawPointer();
+    void  enable();
+    void  disable();
+    virtual void setRawData ( const FMouse::MouseType&
+                            , FKeyboard::keybuffer& );
+    virtual void processEvent (const TimeValue&);
+    void  processQueuedInput();
+    auto  getGpmKeyPressed (bool = true) -> bool;
+    void  drawPointer();
 
   private:
-    // Using-declaration
+    // Constants
+    static constexpr std::size_t MAX_QUEUE_SIZE = 64;
+
+    // Using-declarations
     using FMousePtr = std::unique_ptr<FMouse>;
-    using FMouseDataPtr = std::unique_ptr<FMouseData>;
     using FMouseProtocol = std::vector<FMousePtr>;
+    using MouseQueue = FRingBuffer<FMouseDataPtr, MAX_QUEUE_SIZE>;
 
     // Accessor
-    auto                      findMouseWithType (const FMouse::MouseType&) -> FMouseProtocol::const_iterator;
-    auto                      findMouseWithData() -> FMouseProtocol::const_iterator;
-    auto                      findMouseWithEvent() -> FMouseProtocol::const_iterator;
-    void                      xtermMouse (bool = true) const;
-    void                      enableXTermMouse() const;
-    void                      disableXTermMouse() const;
+    auto  findMouseWithType (const FMouse::MouseType&) const -> FMouseProtocol::const_iterator;
+    auto  findMouseWithData() const -> FMouseProtocol::const_iterator;
+    auto  findMouseWithEvent() const -> FMouseProtocol::const_iterator;
+
+    // Mutators
+    void  xtermMouse (bool = true) const;
+    void  enableXTermMouse() const;
+    void  disableXTermMouse() const;
+    static void  setCurrentMouseEvent (const FMouseDataPtr&);
+    static void  resetCurrentMouseEvent();
 
     // Data member
-    FMouseProtocol            mouse_protocol{};
-    FMouseCommand             event_cmd{};
-    static FMouseData*        current_mouse_event;
-    std::queue<FMouseDataPtr> fmousedata_queue{};
-    FPoint                    zero_point{0, 0};
-    bool                      use_gpm_mouse{false};
-    bool                      use_xterm_mouse{false};
+    FMouseProtocol  mouse_protocol{};
+    FMouseCommand   event_cmd{};
+    MouseQueue      fmousedata_queue{};
+    FPoint          zero_point{0, 0};
+    bool            use_gpm_mouse{false};
+    bool            use_xterm_mouse{false};
 };
 
 // FMouseControl inline functions
 //----------------------------------------------------------------------
-inline FString FMouseControl::getClassName() const
+inline auto FMouseControl::getClassName() const -> FString
 { return "FMouseControl"; }
 
-//----------------------------------------------------------------------
-inline FMouseData* FMouseControl::getCurrentMouseEvent() const
-{ return current_mouse_event; }
-
 //----------------------------------------------------------------------
 inline void FMouseControl::setEventCommand (const FMouseCommand& cmd)
 { event_cmd = cmd; }
 
 //----------------------------------------------------------------------
-inline bool FMouseControl::hasDataInQueue() const
-{ return ! fmousedata_queue.empty(); }
+inline auto FMouseControl::hasDataInQueue() const -> bool
+{ return ! fmousedata_queue.isEmpty(); }
 
 //----------------------------------------------------------------------
 inline void FMouseControl::enableXTermMouse() const
