@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2015-2021 Markus Gans                                      *
+* Copyright 2015-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -75,8 +75,8 @@ void FMenuBar::resetMenu()
 void FMenuBar::hide()
 {
   const auto& wc = getColorTheme();
-  FColor fg = wc->term_fg;
-  FColor bg = wc->term_bg;
+  const auto& fg = wc->term_fg;
+  const auto& bg = wc->term_bg;
   setColor (fg, bg);
   print() << FPoint{1, 1} << FString{getDesktopWidth(), L' '};
   FWindow::hide();
@@ -92,7 +92,7 @@ void FMenuBar::adjustSize()
 //----------------------------------------------------------------------
 void FMenuBar::onKeyPress (FKeyEvent* ev)
 {
-  const auto key = ev->key();
+  const auto& key = ev->key();
 
   if ( isEnterKey(key)
     || key == FKey::Up
@@ -116,16 +116,16 @@ void FMenuBar::onKeyPress (FKeyEvent* ev)
 
     ev->accept();
   }
-  else if ( key == FKey::Left )
-  {
-    selectPrevItem();
-    ev->accept();
-  }
-  else if ( key == FKey::Right )
+  else if ( isFocusNextKey(key) )
   {
     selectNextItem();
     ev->accept();
   }
+  else if ( isFocusPrevKey(key) )
+  {
+    selectPrevItem();
+    ev->accept();
+  }
   else if ( isEscapeKey(key) )
   {
     leaveMenuBar();
@@ -198,10 +198,7 @@ void FMenuBar::onAccel (FAccelEvent* ev)
   unselectItem();
   selectFirstItem();
   getSelectedItem()->setFocus();
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
-
+  drawStatusBarMessage();
   redraw();
   forceTerminalUpdate();
   ev->accept();
@@ -226,7 +223,7 @@ void FMenuBar::init()
   auto r = getRootWidget();
   auto w = r->getWidth();
   // initialize geometry values
-  setGeometry (FPoint{1, 1}, FSize{w, 1}, false);
+  FWindow::setGeometry (FPoint{1, 1}, FSize{w, 1}, false);
   setAlwaysOnTop();
   setMenuBar(this);
   ignorePadding();
@@ -272,13 +269,9 @@ void FMenuBar::selectItem_PostProcessing (FMenuItem* sel_item)
   sel_item->setFocus();
 
   if ( drop_down && sel_item->hasMenu() )
-  {
     openMenu(sel_item);
-  }
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
 
+  drawStatusBarMessage();
   setSelectedItem(sel_item);
   redraw();
   setTerminalUpdates (FVTerm::TerminalUpdate::Start);
@@ -286,47 +279,46 @@ void FMenuBar::selectItem_PostProcessing (FMenuItem* sel_item)
 }
 
 //----------------------------------------------------------------------
-bool FMenuBar::hotkeyMenu (FKeyEvent*& ev)
+auto FMenuBar::hotkeyMenu (FKeyEvent*& ev) -> bool
 {
   for (auto&& item : getItemList())
   {
-    if ( item->isEnabled() )
+    if ( ! item->isEnabled() )
+      continue;
+
+    auto hotkey = item->getHotkey();
+
+    if ( hotkey > 0xff00 && hotkey < 0xff5f )  // full-width character
+      hotkey -= 0xfee0;
+
+    if ( FKey::Meta_offset + FKey(std::tolower(int(hotkey))) == ev->key() )
     {
-      FKey hotkey = item->getHotkey();
-      FKey key = ev->key();
+      const auto& sel_item = getSelectedItem();
 
-      if ( hotkey > 0xff00 && hotkey < 0xff5f )  // full-width character
-        hotkey -= 0xfee0;
+      if ( sel_item && sel_item->hasMenu() )
+        sel_item->getMenu()->unselectItem();
 
-      if ( FKey::Meta_offset + FKey(std::tolower(int(hotkey))) == key )
+      unselectItem();
+
+      if ( item->hasMenu() )
       {
-        auto sel_item = getSelectedItem();
-
-        if ( sel_item && sel_item->hasMenu() )
-          sel_item->getMenu()->unselectItem();
-
-        unselectItem();
-
-        if ( item->hasMenu() )
-        {
-          setTerminalUpdates (FVTerm::TerminalUpdate::Stop);
-          item->setSelected();
-          setSelectedItem(item);
-          item->setFocus();
-          openMenu(item);
-          setTerminalUpdates (FVTerm::TerminalUpdate::Start);
-        }
-        else
-        {
-          setSelectedItem(nullptr);
-          redraw();
-          drop_down = false;
-          item->processClicked();
-        }
-
-        ev->accept();
-        return true;
+        setTerminalUpdates (FVTerm::TerminalUpdate::Stop);
+        item->setSelected();
+        setSelectedItem(item);
+        item->setFocus();
+        openMenu(item);
+        setTerminalUpdates (FVTerm::TerminalUpdate::Start);
       }
+      else
+      {
+        setSelectedItem(nullptr);
+        redraw();
+        drop_down = false;
+        item->processClicked();
+      }
+
+      ev->accept();
+      return true;
     }
   }
 
@@ -342,7 +334,7 @@ void FMenuBar::draw()
 //----------------------------------------------------------------------
 void FMenuBar::drawItems()
 {
-  auto list = getItemList();
+  const auto& list = getItemList();
 
   if ( list.empty() )
     return;
@@ -368,9 +360,14 @@ void FMenuBar::drawItems()
 //----------------------------------------------------------------------
 inline void FMenuBar::drawItem (FMenuItem* menuitem, std::size_t& x)
 {
-  menuText txtdata{};
-  txtdata.startpos = x + 1;
-  txtdata.no_underline = menuitem->getFlags().no_underline;
+  menuText txtdata =
+  {
+    {},
+    x + 1,
+    NOT_SET,
+    bool(menuitem->getFlags().feature.no_underline)
+  };
+
   FString txt{menuitem->getText()};
   std::size_t column_width = getColumnWidth(txt);
   bool is_enabled  = menuitem->isEnabled();
@@ -390,9 +387,7 @@ inline void FMenuBar::drawItem (FMenuItem* menuitem, std::size_t& x)
 
   x += column_width;
 
-  if ( ! is_enabled || is_selected )
-    txtdata.hotkeypos = NOT_SET;
-  else
+  if ( is_enabled && ! is_selected )
     txtdata.hotkeypos = hotkeypos;
 
   setCursorToHotkeyPosition (menuitem, hotkeypos);
@@ -567,10 +562,14 @@ void FMenuBar::adjustItems() const
 //----------------------------------------------------------------------
 void FMenuBar::selectMenuItem (FMenuItem* item)
 {
+  auto focused_widget = getFocusWidget();
+
+  if ( ! item->isEnabled() && focused_widget )
+    focused_widget->unsetFocus();
+
   if ( ! item->isEnabled() || item->isSelected() )
     return;
 
-  auto focused_widget = getFocusWidget();
   unselectItem();
   item->setSelected();
   item->setFocus();
@@ -582,17 +581,17 @@ void FMenuBar::selectMenuItem (FMenuItem* item)
   setSelectedItem(item);
   focus_changed = true;
 
-  if ( item->hasMenu() )
-  {
-    auto menu = item->getMenu();
+  if ( ! item->hasMenu() )
+    return;
 
-    if ( menu->hasSelectedItem() )
-    {
-      menu->unselectItem();
-      menu->redraw();
-      drop_down = true;
-    }
-  }
+  auto menu = item->getMenu();
+
+  if ( ! menu->hasSelectedItem() )
+    return;
+
+  menu->unselectItem();
+  menu->redraw();
+  drop_down = true;
 }
 
 //----------------------------------------------------------------------
@@ -607,16 +606,13 @@ void FMenuBar::openMenu (const FMenuItem* sel_item)
     first_item->setFocus();
 
   menu->redraw();
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
-
+  drawStatusBarMessage();
   redraw();
   drop_down = true;
 }
 
 //----------------------------------------------------------------------
-bool FMenuBar::activateMenu (const FMenuItem* item)
+auto FMenuBar::activateMenu (const FMenuItem* item) -> bool
 {
   if ( ! item->hasMenu() )
     return false;
@@ -631,9 +627,7 @@ bool FMenuBar::activateMenu (const FMenuItem* item)
     if ( first_item )
       first_item->setFocus();
 
-    if ( getStatusBar() )
-      getStatusBar()->drawMessage();
-
+    drawStatusBarMessage();
     redraw();
     menu->redraw();
     drop_down = true;
@@ -643,7 +637,7 @@ bool FMenuBar::activateMenu (const FMenuItem* item)
 }
 
 //----------------------------------------------------------------------
-bool FMenuBar::clickItem (FMenuItem* item)
+auto FMenuBar::clickItem (FMenuItem* item) -> bool
 {
   if ( item->hasMenu() )
     return false;
@@ -678,7 +672,7 @@ void FMenuBar::unselectMenuItem (FMenuItem* item)
 //----------------------------------------------------------------------
 void FMenuBar::mouseDownOverList (const FMouseEvent* ev)
 {
-  auto list = getItemList();
+  const auto& list = getItemList();
 
   if ( list.empty() )
     return;
@@ -716,7 +710,7 @@ void FMenuBar::mouseDownOverList (const FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FMenuBar::mouseUpOverList (const FMouseEvent* ev)
 {
-  auto list = getItemList();
+  const auto& list = getItemList();
 
   if ( list.empty() )
     return;
@@ -753,7 +747,7 @@ void FMenuBar::mouseUpOverList (const FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FMenuBar::mouseMoveOverList (const FMouseEvent& ev)
 {
-  auto list = getItemList();
+  const auto& list = getItemList();
 
   if ( list.empty() )
     return;
@@ -842,10 +836,7 @@ void FMenuBar::leaveMenuBar()
     getStatusBar()->clearMessage();
 
   switchToPrevWindow(this);
-
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
-
+  drawStatusBarMessage();
   mouse_down = false;
 }
 
