@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -22,6 +22,8 @@
 
 #include <cstring>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "final/fc.h"
 #include "final/input/fkey_map.h"
@@ -33,6 +35,21 @@
 namespace finalcut
 {
 
+//----------------------------------------------------------------------
+template <typename CapabilityT, typename StringT>
+constexpr void setTCapString (CapabilityT& cap, StringT&& string)
+{
+  cap = std::forward<StringT>(string);
+}
+
+//----------------------------------------------------------------------
+template <typename CapabilityT, typename StringT>
+constexpr void setTCapStringIfNotSet (CapabilityT& cap, StringT&& string)
+{
+  if ( ! cap )
+    cap = std::forward<StringT>(string);
+}
+
 //----------------------------------------------------------------------
 // class FTermcapQuirks
 //----------------------------------------------------------------------
@@ -105,22 +122,22 @@ void FTermcapQuirks::freebsd()
 {
   // FreeBSD console fixes
 
-  TCAP(t_acs_chars) = \
+  setTCapString ( TCAP(t_acs_chars),
       "-\036.\0370\333"
       "a\260f\370g\361"
       "h\261j\331k\277"
       "l\332m\300n\305"
       "q\304t\303u\264"
       "v\301w\302x\263"
-      "y\363z\362~\371";
+      "y\363z\362~\371" );
 
-    TCAP(t_set_attributes) = \
-        CSI "0"
-            "%?%p1%p6%|%t;1%;"
-            "%?%p2%t;4%;"
-            "%?%p1%p3%|%t;7%;"
-            "%?%p4%t;5%;m"
-            "%?%p9%t\016%e\017%;";
+  setTCapString ( TCAP(t_set_attributes),
+      CSI "0"
+          "%?%p1%p6%|%t;1%;"
+          "%?%p2%t;4%;"
+          "%?%p1%p3%|%t;7%;"
+          "%?%p4%t;5%;m"
+          "%?%p9%t\016%e\017%;" );
 
   FTermcap::attr_without_color = 18;
 }
@@ -130,20 +147,16 @@ void FTermcapQuirks::freebsd()
 void FTermcapQuirks::cygwin()
 {
   // Set invisible cursor for cygwin terminal
-  if ( ! TCAP(t_cursor_invisible) )
-    TCAP(t_cursor_invisible) = CSI "?25l";
+  setTCapStringIfNotSet (TCAP(t_cursor_invisible), CSI "?25l");
 
   // Set visible cursor for cygwin terminal
-  if ( ! TCAP(t_cursor_visible) )
-    TCAP(t_cursor_visible) = CSI "?25h";
+  setTCapStringIfNotSet (TCAP(t_cursor_visible), CSI "?25h");
 
   // Set ansi blink for cygwin terminal
-  if ( ! TCAP(t_enter_blink_mode) )
-    TCAP(t_enter_blink_mode) = CSI "5m";
+  setTCapStringIfNotSet (TCAP(t_enter_blink_mode), CSI "5m");
 
   // Set enable alternate character set for cygwin terminal
-  if ( ! TCAP(t_enable_acs) )
-    TCAP(t_enable_acs) = ESC "(B" ESC ")0";
+  setTCapStringIfNotSet (TCAP(t_enable_acs), ESC "(B" ESC ")0");
 
   // Set background color erase for cygwin terminal
   FTermcap::background_color_erase = true;
@@ -160,45 +173,45 @@ void FTermcapQuirks::linux()
   // Set ansi foreground and background color
   if ( FTerm::getMaxColor() > 8 )
   {
-    TCAP(t_set_a_foreground) = \
-        CSI "3%p1%{8}%m%d%?%p1%{7}%>%t;1%e;22%;m";
-    TCAP(t_set_a_background) = \
-        CSI "4%p1%{8}%m%d%?%p1%{7}%>%t;5%e;25%;m";
+    setTCapString ( TCAP(t_set_a_foreground),
+        CSI "3%p1%{8}%m%d%?%p1%{7}%>%t;1%e;22%;m" );
+    setTCapString ( TCAP(t_set_a_background),
+        CSI "4%p1%{8}%m%d%?%p1%{7}%>%t;5%e;25%;m" );
     // Avoid underline, blink, dim mode and reverse
     FTermcap::attr_without_color = 30;
   }
   else
   {
-    TCAP(t_set_a_foreground) = CSI "3%p1%dm";
-    TCAP(t_set_a_background) = CSI "4%p1%dm";
+    setTCapString (TCAP(t_set_a_foreground), CSI "3%p1%dm");
+    setTCapString (TCAP(t_set_a_background), CSI "4%p1%dm");
     // Avoid underline and dim mode
     FTermcap::attr_without_color = 18;
   }
 
   // Set select graphic rendition attributes
-  TCAP(t_set_attributes) = \
+  setTCapString ( TCAP(t_set_attributes),
       CSI "0"
           "%?%p6%t;1%;"
           "%?%p1%p3%|%t;7%;"
           "%?%p4%t;5%;m"
-          "%?%p9%t\016%e\017%;";
+          "%?%p9%t\016%e\017%;" );
 
-  TCAP(t_enter_alt_charset_mode) = SO;
-  TCAP(t_exit_alt_charset_mode) = SI;
-  TCAP(t_exit_attribute_mode) = CSI "0m" SI;
-  TCAP(t_exit_bold_mode) = CSI "22m";
-  TCAP(t_exit_blink_mode) = CSI "25m";
-  TCAP(t_exit_reverse_mode) = CSI "27m";
-  TCAP(t_exit_secure_mode) = nullptr;
-  TCAP(t_exit_protected_mode) = nullptr;
-  TCAP(t_exit_crossed_out_mode) = nullptr;
-  TCAP(t_orig_pair) = CSI "39;49;25m";
+  setTCapString (TCAP(t_enter_alt_charset_mode), SO);
+  setTCapString (TCAP(t_exit_alt_charset_mode), SI);
+  setTCapString (TCAP(t_exit_attribute_mode), CSI "0m" SI);
+  setTCapString (TCAP(t_exit_bold_mode), CSI "22m");
+  setTCapString (TCAP(t_exit_blink_mode), CSI "25m");
+  setTCapString (TCAP(t_exit_reverse_mode), CSI "27m");
+  setTCapString (TCAP(t_exit_secure_mode), nullptr);
+  setTCapString (TCAP(t_exit_protected_mode), nullptr);
+  setTCapString (TCAP(t_exit_crossed_out_mode), nullptr);
+  setTCapString (TCAP(t_orig_pair), CSI "39;49;25m");
 
   // Avoid underline and dim mode
-  TCAP(t_enter_dim_mode)       = nullptr;
-  TCAP(t_exit_dim_mode)        = nullptr;
-  TCAP(t_enter_underline_mode) = nullptr;
-  TCAP(t_exit_underline_mode)  = nullptr;
+  setTCapString (TCAP(t_enter_dim_mode), nullptr);
+  setTCapString (TCAP(t_exit_dim_mode), nullptr);
+  setTCapString (TCAP(t_enter_underline_mode), nullptr);
+  setTCapString (TCAP(t_exit_underline_mode), nullptr);
 }
 
 //----------------------------------------------------------------------
@@ -208,20 +221,18 @@ void FTermcapQuirks::xterm()
   if ( ! TCAP(t_initialize_color) )
   {
     FTermcap::can_change_color_palette = true;
-    TCAP(t_initialize_color) = \
+    setTCapString ( TCAP(t_initialize_color),
         OSC "4;%p1%d;rgb:"
             "%p2%{255}%*%{1000}%/%2.2X/"
             "%p3%{255}%*%{1000}%/%2.2X/"
-            "%p4%{255}%*%{1000}%/%2.2X" ESC "\\";
+            "%p4%{255}%*%{1000}%/%2.2X" ESC "\\");
   }
 
   // Fallback if "vi" is not found
-  if ( ! TCAP(t_cursor_invisible) )
-    TCAP(t_cursor_invisible) = CSI "?25l";
+  setTCapStringIfNotSet (TCAP(t_cursor_invisible), CSI "?25l");
 
   // Fallback if "ve" is not found
-  if ( ! TCAP(t_cursor_normal) )
-    TCAP(t_cursor_normal) = CSI "?12l" CSI "?25h";
+  setTCapStringIfNotSet (TCAP(t_cursor_normal), CSI "?12l" CSI "?25h");
 }
 
 //----------------------------------------------------------------------
@@ -232,17 +243,17 @@ void FTermcapQuirks::rxvt()
 
   if ( termtype.substr(0, 12) == "rxvt-16color" )
   {
-    TCAP(t_enter_alt_charset_mode) = ESC "(0";
-    TCAP(t_exit_alt_charset_mode)  = ESC "(B";
+    setTCapString (TCAP(t_enter_alt_charset_mode), ESC "(0");
+    setTCapString (TCAP(t_exit_alt_charset_mode), ESC "(B");
   }
 
   // Set ansi foreground and background color
   if ( ! FTermData::getInstance().isTermType(FTermType::urxvt) )
   {
-    TCAP(t_set_a_foreground) = \
-        CSI "%?%p1%{8}%<%t%p1%{30}%+%e%p1%'R'%+%;%dm";
-    TCAP(t_set_a_background) = \
-        CSI "%?%p1%{8}%<%t%p1%'('%+%e%p1%{92}%+%;%dm";
+    setTCapString ( TCAP(t_set_a_foreground),
+        CSI "%?%p1%{8}%<%t%p1%{30}%+%e%p1%'R'%+%;%dm");
+    setTCapString ( TCAP(t_set_a_background),
+        CSI "%?%p1%{8}%<%t%p1%'('%+%e%p1%{92}%+%;%dm");
   }
 }
 
@@ -255,7 +266,7 @@ void FTermcapQuirks::vte()
   FTermcap::attr_without_color = 0;
 
   // set exit underline for gnome terminal
-  TCAP(t_exit_underline_mode) = CSI "24m";
+  setTCapString (TCAP(t_exit_underline_mode), CSI "24m");
 
   if ( fterm_data.getGnomeTerminalID() >= 5300 )  // vte >= 0.53.0
     caModeExtension();
@@ -278,72 +289,46 @@ void FTermcapQuirks::putty()
   FTermcap::attr_without_color = 0;
 
   // Set ansi foreground and background color
-  TCAP(t_set_a_foreground) = \
+  setTCapString ( TCAP(t_set_a_foreground),
       CSI "%?%p1%{8}%<"
           "%t3%p1%d"
           "%e%p1%{16}%<"
           "%t9%p1%{8}%-%d"
-          "%e38;5;%p1%d%;m";
+          "%e38;5;%p1%d%;m" );
 
-  TCAP(t_set_a_background) = \
+  setTCapString ( TCAP(t_set_a_background),
       CSI "%?%p1%{8}%<"
           "%t4%p1%d"
           "%e%p1%{16}%<"
           "%t10%p1%{8}%-%d"
-          "%e48;5;%p1%d%;m";
+          "%e48;5;%p1%d%;m" );
 
-  TCAP(t_set_attributes) = \
+  setTCapString ( TCAP(t_set_attributes),
       CSI "0"
           "%?%p1%p6%|%t;1%;"
           "%?%p5%t;2%;"  // since putty 0.71
           "%?%p2%t;4%;"
           "%?%p1%p3%|%t;7%;"
           "%?%p4%t;5%;m"
-          "%?%p9%t\016%e\017%;";
+          "%?%p9%t\016%e\017%;" );
   // PuTTY 0.71 or higher
-  TCAP(t_enter_dim_mode) = CSI "2m";
+  setTCapString (TCAP(t_enter_dim_mode), CSI "2m");
 
   // PuTTY 0.71 or higher
-  TCAP(t_exit_dim_mode) = CSI "22m";
-
-  if ( ! TCAP(t_clr_bol) )
-    TCAP(t_clr_bol) = CSI "1K";
-
-  if ( ! TCAP(t_orig_pair) )
-    TCAP(t_orig_pair) = CSI "39;49m";
-
-  if ( ! TCAP(t_orig_colors) )
-    TCAP(t_orig_colors) = OSC "R";
-
-  if ( ! TCAP(t_column_address) )
-    TCAP(t_column_address) = CSI "%i%p1%dG";
-
-  if ( ! TCAP(t_row_address) )
-    TCAP(t_row_address) = CSI "%i%p1%dd";
-
-  if ( ! TCAP(t_enable_acs) )
-    TCAP(t_enable_acs) = ESC "(B" ESC ")0";
-
-  if ( ! TCAP(t_enter_am_mode) )
-    TCAP(t_enter_am_mode) = CSI "?7h";
-
-  if ( ! TCAP(t_exit_am_mode) )
-    TCAP(t_exit_am_mode) = CSI "?7l";
-
-  if ( ! TCAP(t_enter_pc_charset_mode) )
-    TCAP(t_enter_pc_charset_mode) = CSI "11m";
-
-  if ( ! TCAP(t_exit_pc_charset_mode) )
-    TCAP(t_exit_pc_charset_mode) = CSI "10m";
-
-  if ( ! TCAP(t_keypad_xmit) )
-    TCAP(t_keypad_xmit) = CSI "?1h" ESC "=";
-
-  if ( ! TCAP(t_keypad_local) )
-    TCAP(t_keypad_local) = CSI "?1l" ESC ">";
-
-  if ( ! TCAP(t_key_mouse) )
-    TCAP(t_key_mouse) = CSI "M";
+  setTCapString (TCAP(t_exit_dim_mode), CSI "22m");
+  setTCapStringIfNotSet (TCAP(t_clr_bol), CSI "1K");
+  setTCapStringIfNotSet (TCAP(t_orig_pair), CSI "39;49m");
+  setTCapStringIfNotSet (TCAP(t_orig_colors), OSC "R");
+  setTCapStringIfNotSet (TCAP(t_column_address), CSI "%i%p1%dG");
+  setTCapStringIfNotSet (TCAP(t_row_address), CSI "%i%p1%dd");
+  setTCapStringIfNotSet (TCAP(t_enable_acs), ESC "(B" ESC ")0");
+  setTCapStringIfNotSet (TCAP(t_enter_am_mode), CSI "?7h");
+  setTCapStringIfNotSet (TCAP(t_exit_am_mode), CSI "?7l");
+  setTCapStringIfNotSet (TCAP(t_enter_pc_charset_mode), CSI "11m");
+  setTCapStringIfNotSet (TCAP(t_exit_pc_charset_mode), CSI "10m");
+  setTCapStringIfNotSet (TCAP(t_keypad_xmit), CSI "?1h" ESC "=");
+  setTCapStringIfNotSet (TCAP(t_keypad_local), CSI "?1l" ESC ">");
+  setTCapStringIfNotSet (TCAP(t_key_mouse), CSI "M");
 }
 
 //----------------------------------------------------------------------
@@ -353,10 +338,10 @@ void FTermcapQuirks::teraterm()
   FTermcap::eat_nl_glitch = true;
 
   // Tera Term color settings
-  TCAP(t_set_a_foreground) = CSI "38;5;%p1%dm";
-  TCAP(t_set_a_background) = CSI "48;5;%p1%dm";
-  TCAP(t_exit_attribute_mode) = CSI "0m" SI;
-  TCAP(t_orig_pair) = CSI "39;49m";
+  setTCapString (TCAP(t_set_a_foreground), CSI "38;5;%p1%dm");
+  setTCapString (TCAP(t_set_a_background), CSI "48;5;%p1%dm");
+  setTCapString (TCAP(t_exit_attribute_mode), CSI "0m" SI);
+  setTCapString (TCAP(t_orig_pair), CSI "39;49m");
 }
 
 //----------------------------------------------------------------------
@@ -366,78 +351,51 @@ void FTermcapQuirks::sunConsole()
   FTermcap::eat_nl_glitch = true;
 
   // Sun Microsystems workstation console parameter cursor control
-  TCAP(t_parm_up_cursor) = CSI "%p1%dA";
-  TCAP(t_parm_down_cursor) = CSI "%p1%dB";
-  TCAP(t_parm_right_cursor) = CSI "%p1%dC";
-  TCAP(t_parm_left_cursor) = CSI "%p1%dD";
+  setTCapString (TCAP(t_parm_up_cursor), CSI "%p1%dA");
+  setTCapString (TCAP(t_parm_down_cursor), CSI "%p1%dB");
+  setTCapString (TCAP(t_parm_right_cursor), CSI "%p1%dC");
+  setTCapString (TCAP(t_parm_left_cursor), CSI "%p1%dD");
   auto& fkey_cap_table = FKeyMap::getKeyCapMap();
 
   // Sun Microsystems workstation console keys
+  std::vector<std::pair<std::string, std::string>> sun_console_keys = \
+  {
+    {"K2", CSI "218z"},  // center of keypad
+    {"kb", "\b"},  // backspace key
+    {"kD", "\177"},  // delete-character key
+    {"@7", CSI "220z"},  // end key
+    {"k;", CSI "233z"},  // F10 function key
+    {"F1", CSI "234z"},  // F11 function key
+    {"F2", CSI "235z"},  // F12 function key
+    {"kh", CSI "214z"},  // home key
+    {"kI", CSI "247z"},  // insert-character key
+    {"kN", CSI "222z"},  // next-page key
+    {"%7", CSI "194z"},  // options key
+    {"kP", CSI "216z"},  // prev-page key
+    {"&5", CSI "193z"},  // resume key
+    {"&8", CSI "195z"},  // undo key
+    {"K2", CSI "218z"},  // center of keypad
+    {"kDx", CSI "249z"},  // keypad delete
+    {"@8x", CSI "250z"},  // enter/send key
+    {"KP1", CSI "212z"},  // keypad slash
+    {"KP2", CSI "213z"},  // keypad asterisk
+    {"KP3", CSI "254z"},  // keypad minus sign
+    {"KP4", CSI "253z"},  // keypad plus sign
+  };
+
   for (std::size_t i{0}; fkey_cap_table[i].tname[0] != 0; i++)
   {
-    if ( std::strncmp(fkey_cap_table[i].tname, "K2", 2) == 0 )
-      fkey_cap_table[i].string = CSI "218z";  // center of keypad
+    for (const auto& key : sun_console_keys)
+    {
+      const std::string& tname = key.first;
+      const std::string& string = key.second;
 
-    if ( std::strncmp(fkey_cap_table[i].tname, "kb", 2) == 0 )
-      fkey_cap_table[i].string = "\b";  // backspace key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kD", 2) == 0
-      && stringLength(fkey_cap_table[i].tname) == 2 )
-      fkey_cap_table[i].string = "\177";  // delete-character key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "@7", 2) == 0 )
-      fkey_cap_table[i].string = CSI "220z";  // end key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "k;", 2) == 0 )
-      fkey_cap_table[i].string = CSI "233z";  // F10 function key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "F1", 2) == 0 )
-      fkey_cap_table[i].string = CSI "234z";  // F11 function key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "F2", 2) == 0 )
-      fkey_cap_table[i].string = CSI "235z";  // F12 function key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kh", 2) == 0 )
-      fkey_cap_table[i].string = CSI "214z";  // home key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kI", 2) == 0 )
-      fkey_cap_table[i].string = CSI "247z";  // insert-character key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kN", 2) == 0 )
-      fkey_cap_table[i].string = CSI "222z";  // next-page key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "%7", 2) == 0 )
-      fkey_cap_table[i].string = CSI "194z";  // options key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kP", 2) == 0 )
-      fkey_cap_table[i].string = CSI "216z";  // prev-page key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "&5", 2) == 0 )
-      fkey_cap_table[i].string = CSI "193z";  // resume key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "&8", 2) == 0 )
-      fkey_cap_table[i].string = CSI "195z";  // undo key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "K2", 2) == 0 )
-      fkey_cap_table[i].string = CSI "218z";  // center of keypad
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "kDx", 3) == 0 )
-      fkey_cap_table[i].string = CSI "249z";  // keypad delete
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "@8x", 3) == 0 )
-      fkey_cap_table[i].string = CSI "250z";  // enter/send key
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "KP1", 3) == 0 )
-      fkey_cap_table[i].string = CSI "212z";  // keypad slash
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "KP2", 3) == 0 )
-      fkey_cap_table[i].string = CSI "213z";  // keypad asterisk
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "KP3", 3) == 0 )
-      fkey_cap_table[i].string = CSI "254z";  // keypad minus sign
-
-    if ( std::strncmp(fkey_cap_table[i].tname, "KP4", 3) == 0 )
-      fkey_cap_table[i].string = CSI "253z";  // keypad plus sign
+      if ( std::memcmp(fkey_cap_table[i].tname, tname.c_str(), tname.size()) == 0
+        && stringLength(fkey_cap_table[i].tname) == tname.size() )
+      {
+        fkey_cap_table[i].string = string.c_str();
+      }
+    }
   }
 }
 
@@ -451,19 +409,19 @@ void FTermcapQuirks::screen()
 
     if ( FTermData::getInstance().isTermType(FTermType::tmux) )
     {
-      TCAP(t_initialize_color) = \
+      setTCapString ( TCAP(t_initialize_color),
           ESC "Ptmux;" ESC OSC "4;%p1%d;rgb:"
           "%p2%{255}%*%{1000}%/%2.2X/"
           "%p3%{255}%*%{1000}%/%2.2X/"
-          "%p4%{255}%*%{1000}%/%2.2X" BEL ESC "\\";
+          "%p4%{255}%*%{1000}%/%2.2X" BEL ESC "\\" );
     }
     else
     {
-      TCAP(t_initialize_color) = \
+      setTCapString ( TCAP(t_initialize_color),
           ESC "P" OSC "4;%p1%d;rgb:"
           "%p2%{255}%*%{1000}%/%2.2X/"
           "%p3%{255}%*%{1000}%/%2.2X/"
-          "%p4%{255}%*%{1000}%/%2.2X" BEL ESC "\\";
+          "%p4%{255}%*%{1000}%/%2.2X" BEL ESC "\\" );
     }
   }
 }
@@ -480,35 +438,30 @@ void FTermcapQuirks::general()
     FTermcap::attr_without_color = 0;
 
   // Fallback if "AF" is not found
-  if ( ! TCAP(t_set_a_foreground) )
-    TCAP(t_set_a_foreground) = CSI "3%p1%dm";
+  setTCapStringIfNotSet (TCAP(t_set_a_foreground), CSI "3%p1%dm");
 
   // Fallback if "AB" is not found
-  if ( ! TCAP(t_set_a_background) )
-    TCAP(t_set_a_background) = CSI "4%p1%dm";
+  setTCapStringIfNotSet (TCAP(t_set_a_background), CSI "4%p1%dm");
 
   // Fallback if "Ic" is not found
   if ( ! TCAP(t_initialize_color) )
   {
     FTermcap::can_change_color_palette = true;
-    TCAP(t_initialize_color) = \
+    setTCapString ( TCAP(t_initialize_color),
         OSC "P%p1%x"
             "%p2%{255}%*%{1000}%/%02x"
             "%p3%{255}%*%{1000}%/%02x"
-            "%p4%{255}%*%{1000}%/%02x";
+            "%p4%{255}%*%{1000}%/%02x" );
   }
 
   // Fallback if "ti" is not found
-  if ( ! TCAP(t_enter_ca_mode) )
-    TCAP(t_enter_ca_mode) = ESC "7" CSI "?47h";
+  setTCapStringIfNotSet (TCAP(t_enter_ca_mode), ESC "7" CSI "?47h");
 
   // Fallback if "te" is not found
-  if ( ! TCAP(t_exit_ca_mode) )
-    TCAP(t_exit_ca_mode) = CSI "?47l" ESC "8" CSI "m";
+  setTCapStringIfNotSet (TCAP(t_exit_ca_mode), CSI "?47l" ESC "8" CSI "m");
 
   // Set ansi move if "cm" is not found
-  if ( ! TCAP(t_cursor_address) )
-    TCAP(t_cursor_address) = CSI "%i%p1%d;%p2%dH";
+  setTCapStringIfNotSet (TCAP(t_cursor_address), CSI "%i%p1%d;%p2%dH");
 }
 
 //----------------------------------------------------------------------
@@ -519,7 +472,7 @@ inline void FTermcapQuirks::caModeExtension()
   {
     // Save the cursor position, enter alternate screen buffer
     // and save xterm icon and window title on stack
-    TCAP(t_enter_ca_mode) = CSI "?1049h" CSI "22;0;0t";
+    setTCapString (TCAP(t_enter_ca_mode), CSI "?1049h" CSI "22;0;0t");
   }
 
   if ( TCAP(t_exit_ca_mode)
@@ -527,7 +480,7 @@ inline void FTermcapQuirks::caModeExtension()
   {
     // Use normal screen buffer, restore the cursor position
     // and restore xterm icon and window title from stack
-    TCAP(t_exit_ca_mode) = CSI "?1049l" CSI "23;0;0t";
+    setTCapString (TCAP(t_exit_ca_mode), CSI "?1049l" CSI "23;0;0t");
   }
 }
 
@@ -536,20 +489,20 @@ void FTermcapQuirks::ecma48()
 {
   // Test for standard ECMA-48 (ANSI X3.64) terminal
   if ( ! TCAP(t_exit_underline_mode)
-    || std::strncmp(TCAP(t_exit_underline_mode), CSI "24m", 5) != 0 )
+    || std::memcmp(TCAP(t_exit_underline_mode), CSI "24m", 5) != 0 )
     return;
 
   // Seems to be a ECMA-48 (ANSI X3.64) compatible terminal
-  TCAP(t_enter_dbl_underline_mode) = CSI "21m";  // Leaves single underlined too
-  TCAP(t_exit_dbl_underline_mode) = CSI "24m";
-  TCAP(t_exit_bold_mode) = CSI "22m";  // Exit dim, too
-  TCAP(t_exit_dim_mode) = CSI "22m";
-  TCAP(t_exit_underline_mode) = CSI "24m";
-  TCAP(t_exit_blink_mode) = CSI "25m";
-  TCAP(t_exit_reverse_mode) = CSI "27m";
-  TCAP(t_exit_secure_mode) = CSI "28m";
-  TCAP(t_enter_crossed_out_mode) = CSI "9m";
-  TCAP(t_exit_crossed_out_mode) = CSI "29m";
+  setTCapString (TCAP(t_enter_dbl_underline_mode), CSI "21m");  // Leaves single underlined too
+  setTCapString (TCAP(t_exit_dbl_underline_mode), CSI "24m");
+  setTCapString (TCAP(t_exit_bold_mode), CSI "22m");  // Exit dim, too
+  setTCapString (TCAP(t_exit_dim_mode), CSI "22m");
+  setTCapString (TCAP(t_exit_underline_mode), CSI "24m");
+  setTCapString (TCAP(t_exit_blink_mode), CSI "25m");
+  setTCapString (TCAP(t_exit_reverse_mode), CSI "27m");
+  setTCapString (TCAP(t_exit_secure_mode), CSI "28m");
+  setTCapString (TCAP(t_enter_crossed_out_mode), CSI "9m");
+  setTCapString (TCAP(t_exit_crossed_out_mode), CSI "29m");
 }
 
 }  // namespace finalcut
