@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -20,11 +20,9 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
-#if defined(__CYGWIN__)
-  #include "final/fconfig.h"  // includes _GNU_SOURCE for fd_set
-#endif
-
 #include <array>
+#include <memory>
+#include <string>
 
 #include "final/fapplication.h"
 #include "final/fc.h"
@@ -59,7 +57,7 @@ FTermXTerminal::FTermXTerminal() = default;
 //----------------------------------------------------------------------
 auto FTermXTerminal::getInstance() -> FTermXTerminal&
 {
-  static const auto& xterm = make_unique<FTermXTerminal>();
+  static const auto& xterm = std::make_unique<FTermXTerminal>();
   return *xterm;
 }
 
@@ -165,6 +163,17 @@ void FTermXTerminal::setMouseSupport (bool enable)
     disableXTermMouse();
 }
 
+//----------------------------------------------------------------------
+void FTermXTerminal::setFocusSupport (bool enable)
+{
+  // activate/deactivate the terminal focus events
+
+  if ( enable )
+    enableXTermFocus();
+  else
+    disableXTermFocus();
+}
+
 //----------------------------------------------------------------------
 void FTermXTerminal::metaSendsESC (bool enable)
 {
@@ -562,7 +571,7 @@ inline void FTermXTerminal::setXTermDefaultsMouseCursor()
 }
 
 //----------------------------------------------------------------------
-inline bool FTermXTerminal::canSetXTermBackground() const
+inline auto FTermXTerminal::canSetXTermBackground() const -> bool
 {
   static const auto& fterm_data = FTermData::getInstance();
   return ( xterm_default_colors
@@ -581,13 +590,24 @@ void FTermXTerminal::resetXTermColorMap() const
   {
     FTerm::paddingPrint (ESC "c");  // Full Reset (RIS)
   }
+  else if ( canResetColor()
+         && FTermData::getInstance().isTermType(FTermType::stterm) )
+  {
+    for (int c{0}; c < 16; c++)
+    {
+      oscPrefix();
+      FTerm::paddingPrintf (OSC "104;%d" BEL, c);
+      oscPostfix();
+    }
+  }
   else if ( canResetColor() )
   {
     oscPrefix();
     FTerm::paddingPrint (OSC "104" BEL);
     oscPostfix();
-    std::fflush(stdout);
   }
+
+  std::fflush(stdout);
 }
 
 //----------------------------------------------------------------------
@@ -675,7 +695,7 @@ void FTermXTerminal::resetXTermHighlightBackground() const
 }
 
 //----------------------------------------------------------------------
-bool FTermXTerminal::canResetColor() const
+auto FTermXTerminal::canResetColor() const -> bool
 {
   static const auto& fterm_data = FTermData::getInstance();
 
@@ -720,7 +740,7 @@ void FTermXTerminal::oscPostfix() const
 }
 
 //----------------------------------------------------------------------
-FString FTermXTerminal::captureXTermFont() const
+auto FTermXTerminal::captureXTermFont() const -> FString
 {
   static const auto& fterm_data = FTermData::getInstance();
 
@@ -730,38 +750,15 @@ FString FTermXTerminal::captureXTermFont() const
     return {};
   }
 
-  fd_set ifds{};
-  struct timeval tv{};
-  const int stdin_no = FTermios::getStdIn();
-
   // Querying the terminal font
   oscPrefix();
   FTerm::paddingPrint (OSC "50;?" BEL);
   oscPostfix();
   std::fflush(stdout);
-  FD_ZERO(&ifds);
-  FD_SET(stdin_no, &ifds);
-  tv.tv_sec  = 0;
-  tv.tv_usec = 150000;  // 150 ms
-
-  // Read the terminal answer
-  if ( select(stdin_no + 1, &ifds, nullptr, nullptr, &tv) < 1 )
-    return {};
-
   std::array<char, 150> temp{};
-  std::size_t pos{0};
-
-  do
-  {
-    std::size_t bytes_free = temp.size() - pos - 1;
-    const ssize_t bytes = read(stdin_no, &temp[pos], bytes_free);
-
-    if ( bytes <= 0 )
-      break;
-
-    pos += std::size_t(bytes);
-  }
-  while ( pos < temp.size() && ! std::strchr(temp.data(), '\a') );
+  // BEL = \a = bell character
+  auto isWithout_BEL = [] (const auto& t) { return ! std::strchr(t.data(), '\a'); };
+  auto pos = captureTerminalInput(temp, 150'000, isWithout_BEL);
 
   if ( pos > 5 && temp[0] == ESC[0] && temp[1] == ']' && temp[2] == '5'
                && temp[3] == '0' && temp[4] == ';' )
@@ -781,41 +778,18 @@ FString FTermXTerminal::captureXTermFont() const
 }
 
 //----------------------------------------------------------------------
-FString FTermXTerminal::captureXTermTitle() const
+auto FTermXTerminal::captureXTermTitle() const -> FString
 {
   if ( FTermData::getInstance().isTermType(FTermType::kde_konsole) )
     return {};
 
-  fd_set ifds{};
-  struct timeval tv{};
-  const int stdin_no{FTermios::getStdIn()};
-
   // Report window title
   FTerm::paddingPrint (CSI "21t");
   std::fflush(stdout);
-  FD_ZERO(&ifds);
-  FD_SET(stdin_no, &ifds);
-  tv.tv_sec  = 0;
-  tv.tv_usec = 150000;  // 150 ms
-
-  // read the terminal answer
-  if ( select (stdin_no + 1, &ifds, nullptr, nullptr, &tv) < 1 )
-    return {};
-
   std::array<char, 512> temp{};
-  std::size_t pos{0};
-
-  do
-  {
-    std::size_t bytes_free = temp.size() - pos - 1;
-    const ssize_t bytes = read(stdin_no, &temp[pos], bytes_free);
-
-    if ( bytes <= 0 )
-      break;
-
-    pos += std::size_t(bytes);
-  }
-  while ( pos < temp.size() && std::strstr(temp.data(), ESC "\\") == nullptr );
+  // ST = ESC + \ = string terminator
+  auto isWithout_ST  = [] (const auto& t) { return ! std::strstr(t.data(), ESC "\\"); };
+  auto pos = captureTerminalInput(temp, 150'000, isWithout_ST);
 
   if ( pos > 6 && temp[0] == ESC[0] && temp[1] == ']' && temp[2] == 'l' )
   {
@@ -871,6 +845,32 @@ void FTermXTerminal::disableXTermMouse()
   mouse_support = false;
 }
 
+//----------------------------------------------------------------------
+void FTermXTerminal::enableXTermFocus()
+{
+  // Activate terminal focus event
+
+  if ( focus_support )
+    return;  // The terminal focus event is already activated
+
+  FTerm::paddingPrint (CSI "?1004h");  // enable send FocusIn/FocusOut
+  std::fflush(stdout);
+  focus_support = true;
+}
+
+//----------------------------------------------------------------------
+void FTermXTerminal::disableXTermFocus()
+{
+  // Deactivate terminal focus event
+
+  if ( ! focus_support )
+    return;  // The terminal focus event was already deactivated
+
+  FTerm::paddingPrint (CSI "?1004l");  // disable Send FocusIn/FocusOut
+  std::fflush(stdout);
+  focus_support = false;
+}
+
 //----------------------------------------------------------------------
 void FTermXTerminal::enableXTermMetaSendsESC()
 {
