@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2012-2021 Markus Gans                                      *
+* Copyright 2012-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -26,6 +26,7 @@
 #include "final/fapplication.h"
 #include "final/fevent.h"
 #include "final/fwidgetcolors.h"
+#include "final/fwidget_functions.h"
 #include "final/util/flog.h"
 #include "final/util/fpoint.h"
 #include "final/util/fsize.h"
@@ -62,34 +63,38 @@ FLineEdit::FLineEdit (const FString& txt, FWidget* parent)
 //----------------------------------------------------------------------
 FLineEdit::~FLineEdit()  // destructor
 {
+  if ( input_type == InputType::Password )  // Zero password in memory
+    for (auto&& ch : text)
+      ch = '\0';
+
   if ( ! insert_mode )
     FVTerm::getFOutput()->setCursor(CursorMode::Insert);
 }
 
 // FLineEdit operators
 //----------------------------------------------------------------------
-FLineEdit& FLineEdit::operator = (const FString& s)
+auto FLineEdit::operator = (const FString& s) -> FLineEdit&
 {
   setText(s);
   return *this;
 }
 
 //----------------------------------------------------------------------
-FLineEdit& FLineEdit::operator << (UniChar c)
+auto FLineEdit::operator << (UniChar c) -> FLineEdit&
 {
   setText(text + static_cast<wchar_t>(c));
   return *this;
 }
 
 //----------------------------------------------------------------------
-FLineEdit& FLineEdit::operator << (const wchar_t c)
+auto FLineEdit::operator << (const wchar_t c) -> FLineEdit&
 {
   setText(text + c);
   return *this;
 }
 
 //----------------------------------------------------------------------
-const FLineEdit& FLineEdit::operator >> (FString& s) const
+auto FLineEdit::operator >> (FString& s) const -> const FLineEdit&
 {
   s += text;
   return *this;
@@ -98,7 +103,7 @@ const FLineEdit& FLineEdit::operator >> (FString& s) const
 
 // public methods of FLineEdit
 //----------------------------------------------------------------------
-bool FLineEdit::setEnable (bool enable)
+auto FLineEdit::setEnable (bool enable) -> bool
 {
   FWidget::setEnable(enable);
   resetColors();
@@ -106,21 +111,13 @@ bool FLineEdit::setEnable (bool enable)
 }
 
 //----------------------------------------------------------------------
-bool FLineEdit::setFocus (bool enable)
-{
-  FWidget::setFocus(enable);
-  resetColors();
-  return enable;
-}
-
-//----------------------------------------------------------------------
-bool FLineEdit::setShadow (bool enable)
+auto FLineEdit::setShadow (bool enable) -> bool
 {
   return setWidgetShadow(this, enable);
 }
 
 //----------------------------------------------------------------------
-bool FLineEdit::setReadOnly (bool enable)
+auto FLineEdit::setReadOnly (bool enable) -> bool
 {
   if ( enable )
     unsetVisibleCursor();
@@ -130,6 +127,17 @@ bool FLineEdit::setReadOnly (bool enable)
   return (read_only = enable);
 }
 
+//----------------------------------------------------------------------
+void FLineEdit::setAlignment (Align align) noexcept
+{
+  if ( align != Align::Left
+    && align != Align::Center
+    && align != Align::Right )
+    alignment = Align::Left;
+  else
+    alignment = align;
+}
+
 //----------------------------------------------------------------------
 void FLineEdit::setText (const FString& txt)
 {
@@ -154,6 +162,33 @@ void FLineEdit::setText (const FString& txt)
   }
 }
 
+//----------------------------------------------------------------------
+void FLineEdit::inputText (const FString& input)
+{
+  if ( insert_mode )
+    text.insert(input, cursor_pos);
+  else
+    text.overwrite(input, cursor_pos);
+
+  print_text = ( isPasswordField() ) ? getPasswordText() : text;
+
+  if ( isShown() )
+  {
+    if ( ! isReadOnly() )
+      cursor_pos += input.getLength();
+
+    adjustTextOffset();
+  }
+
+  processChanged();
+}
+
+//----------------------------------------------------------------------
+void FLineEdit::deletesCharacter()
+{
+  deleteCurrentCharacter();
+}
+
 //----------------------------------------------------------------------
 void FLineEdit::setMaxLength (std::size_t max)
 {
@@ -192,6 +227,32 @@ void FLineEdit::setCursorPosition (std::size_t pos)
     adjustTextOffset();
 }
 
+//----------------------------------------------------------------------
+void FLineEdit::moveCursorToBegin()
+{
+  cursorHome();
+}
+
+//----------------------------------------------------------------------
+void FLineEdit::moveCursorToEnd()
+{
+  cursorEnd();
+}
+
+//----------------------------------------------------------------------
+void FLineEdit::stepCursorForward (std::size_t steps)
+{
+  while ( steps-- )
+    cursorRight();
+}
+
+//----------------------------------------------------------------------
+void FLineEdit::stepCursorBackward (std::size_t steps)
+{
+  while ( steps-- )
+    cursorLeft();
+}
+
 //----------------------------------------------------------------------
 void FLineEdit::setLabelText (const FString& ltxt)
 {
@@ -200,6 +261,17 @@ void FLineEdit::setLabelText (const FString& ltxt)
   adjustLabel();
 }
 
+//----------------------------------------------------------------------
+void FLineEdit::setOverwriteMode (bool overwrite)
+{
+  insert_mode = ! overwrite;
+
+  if ( insert_mode )
+    FVTerm::getFOutput()->setCursor(CursorMode::Insert);     // Insert mode
+  else
+    FVTerm::getFOutput()->setCursor(CursorMode::Overwrite);  // Overtype mode
+}
+
 //----------------------------------------------------------------------
 void FLineEdit::setLabelOrientation (const LabelOrientation o)
 {
@@ -261,7 +333,7 @@ void FLineEdit::hide()
     label->hide();
 
   FWidget::hide();
-  const FSize shadow = hasShadow() ? FSize{1, 1} : FSize{0, 0};
+  const auto& shadow = hasShadow() ? FSize{1, 1} : FSize{0, 0};
   hideArea (getSize() + shadow);
 }
 
@@ -284,11 +356,11 @@ void FLineEdit::onKeyPress (FKeyEvent* ev)
     return;
 
   const auto key = ev->key();
-  const auto& entry = key_map[key];
+  const auto& iter = key_map.find(key);
 
-  if ( entry )
+  if ( iter != key_map.end() )
   {
-    entry();
+    iter->second();
     ev->accept();
   }
   else if ( key == FKey::Tab || key == FKey::Back_tab
@@ -338,7 +410,7 @@ void FLineEdit::onMouseDown (FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FLineEdit::onMouseUp (FMouseEvent*)
 {
-  if ( drag_scroll != DragScrollMode::None )
+  if ( isDragging(drag_scroll) )
   {
     delOwnTimers();
     drag_scroll = DragScrollMode::None;
@@ -481,8 +553,10 @@ void FLineEdit::onHide (FHideEvent*)
 }
 
 //----------------------------------------------------------------------
-void FLineEdit::onFocusIn (FFocusEvent*)
+void FLineEdit::onFocusIn (FFocusEvent* in_ev)
 {
+  resetColors();
+
   if ( ! isReadOnly() )
   {
     if ( insert_mode )
@@ -491,21 +565,18 @@ void FLineEdit::onFocusIn (FFocusEvent*)
       FVTerm::getFOutput()->setCursor(CursorMode::Overwrite);
   }
 
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
+  FWidget::onFocusIn(in_ev);
 }
 
 //----------------------------------------------------------------------
-void FLineEdit::onFocusOut (FFocusEvent*)
+void FLineEdit::onFocusOut (FFocusEvent* out_ev)
 {
-  if ( getStatusBar() )
-  {
-    getStatusBar()->clearMessage();
-    getStatusBar()->drawMessage();
-  }
+  resetColors();
 
   if ( ! insert_mode && ! isReadOnly() )
     FVTerm::getFOutput()->setCursor(CursorMode::Insert);
+
+  FWidget::onFocusOut(out_ev);
 }
 
 
@@ -566,20 +637,23 @@ void FLineEdit::init()
 //----------------------------------------------------------------------
 inline void FLineEdit::mapKeyFunctions()
 {
-  key_map[FKey::Left]      = [this] { cursorLeft(); };
-  key_map[FKey::Right]     = [this] { cursorRight(); };
-  key_map[FKey::Home]      = [this] { cursorHome(); };
-  key_map[FKey::End]       = [this] { cursorEnd(); };
-  key_map[FKey::Del_char]  = [this] { deleteCurrentCharacter(); };
-  key_map[FKey::Erase]     = [this] { deletePreviousCharacter(); };
-  key_map[FKey::Backspace] = [this] { deletePreviousCharacter(); };
-  key_map[FKey::Insert]    = [this] { switchInsertMode(); };
-  key_map[FKey::Return]    = [this] { acceptInput(); };
-  key_map[FKey::Enter]     = [this] { acceptInput(); };
+  key_map =
+  {
+    { FKey::Left      , [this] { cursorLeft(); } },
+    { FKey::Right     , [this] { cursorRight(); } },
+    { FKey::Home      , [this] { cursorHome(); } },
+    { FKey::End       , [this] { cursorEnd(); } },
+    { FKey::Del_char  , [this] { deleteCurrentCharacter(); } },
+    { FKey::Erase     , [this] { deletePreviousCharacter(); } },
+    { FKey::Backspace , [this] { deletePreviousCharacter(); } },
+    { FKey::Insert    , [this] { switchInsertMode(); } },
+    { FKey::Return    , [this] { acceptInput(); } },
+    { FKey::Enter     , [this] { acceptInput(); } }
+  };
 }
 
 //----------------------------------------------------------------------
-bool FLineEdit::hasHotkey() const
+auto FLineEdit::hasHotkey() const -> bool
 {
   if ( label_text.isEmpty() )
     return false;
@@ -587,6 +661,12 @@ bool FLineEdit::hasHotkey() const
   return label_text.includes('&');
 }
 
+//----------------------------------------------------------------------
+auto FLineEdit::getAlignOffset (const std::size_t length) const -> std::size_t
+{
+  return finalcut::getAlignOffset (alignment, getWidth(), length);
+}
+
 //----------------------------------------------------------------------
 void FLineEdit::draw()
 {
@@ -597,24 +677,13 @@ void FLineEdit::draw()
     adjustTextOffset();
 
   drawInputField();
-
-  if ( getFlags().focus && getStatusBar() )
-  {
-    const auto& msg = getStatusbarMessage();
-    const auto& curMsg = getStatusBar()->getMessage();
-
-    if ( curMsg != msg )
-    {
-      getStatusBar()->setMessage(msg);
-      getStatusBar()->drawMessage();
-    }
-  }
+  updateStatusbar (this);
 }
 
 //----------------------------------------------------------------------
 void FLineEdit::drawInputField()
 {
-  const bool isActiveFocus = getFlags().active && getFlags().focus;
+  const bool isActiveFocus = getFlags().feature.active && getFlags().focus.focus;
   print() << FPoint{1, 1};
 
   if ( FVTerm::getFOutput()->isMonochron() )
@@ -636,7 +705,9 @@ void FLineEdit::drawInputField()
   if ( isActiveFocus && FVTerm::getFOutput()->getMaxColor() < 16 )
     setBold();
 
-  const std::size_t text_offset_column = [this] ()
+  std::size_t text_offset_column;
+  std::size_t x_pos;
+  std::tie(text_offset_column, x_pos) = [this] ()
   {
     switch ( input_type )
     {
@@ -647,16 +718,14 @@ void FLineEdit::drawInputField()
         return printPassword();
 
       default:
-        break;
+        throw std::invalid_argument{"Invalid input type"};
     }
-
-    return std::size_t(0);
   }();
 
-  while ( x_pos + 1 < getWidth() )
+  if ( x_pos + align_offset + 1 < getWidth() )
   {
-    print (' ');
-    x_pos++;
+    const auto trailing_spaces = getWidth() - x_pos - align_offset - 1;
+    print(FString{trailing_spaces, L' '});
   }
 
   if ( isActiveFocus && FVTerm::getFOutput()->getMaxColor() < 16 )
@@ -668,47 +737,56 @@ void FLineEdit::drawInputField()
     setUnderline(false);
   }
 
-  if ( getFlags().shadow )
+  if ( getFlags().shadow.shadow )
     drawShadow(this);
 
   // set the cursor to the insert pos.
   const auto cursor_pos_column = getCursorColumnPos();
-  const auto xpos = int(2 + cursor_pos_column
+  const auto xpos = int(2 + align_offset
+                          + cursor_pos_column
                           - text_offset_column
                           + char_width_offset);
   setCursorPos ({xpos, 1});
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FLineEdit::printTextField()
+inline auto FLineEdit::printTextField() -> std::tuple<std::size_t, std::size_t>
 {
   const std::size_t text_offset_column = getColumnWidth (print_text, text_offset);
   const std::size_t start_column = text_offset_column - char_width_offset + 1;
   const FString& show_text = \
       getColumnSubString(print_text, start_column, getWidth() - 2);
+  const auto x_pos = getColumnWidth(show_text);
+  align_offset = getAlignOffset(x_pos + 2);
+
+  if ( align_offset > 0 )
+    print (FString{align_offset, L' '});  // leading spaces
 
   if ( ! show_text.isEmpty() )
     print (show_text);
 
-  x_pos = getColumnWidth(show_text);
-  return text_offset_column;
+  return std::make_tuple (text_offset_column, x_pos);
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FLineEdit::printPassword()
+inline auto FLineEdit::printPassword() -> std::tuple<std::size_t, std::size_t>
 {
   const std::size_t text_offset_column = text_offset;
   const FString show_text{print_text.mid(1 + text_offset, getWidth() - 2)};
+  const auto x_pos = show_text.getLength();
+  align_offset = getAlignOffset(x_pos + 2);
+
+  if ( align_offset > 0 )
+    print (FString{align_offset, ' '});  // leading spaces
 
   if ( ! show_text.isEmpty() )
-    print() << FString{show_text.getLength(), UniChar::Bullet};  // •
+    print() << FString{x_pos, UniChar::Bullet};  // •
 
-  x_pos = show_text.getLength();
-  return text_offset_column;
+  return std::make_tuple (text_offset_column, x_pos);
 }
 
 //----------------------------------------------------------------------
-inline std::size_t FLineEdit::getCursorColumnPos() const
+inline auto FLineEdit::getCursorColumnPos() const -> std::size_t
 {
   if ( input_type == InputType::Textfield )
     return getColumnWidth (print_text, cursor_pos);
@@ -720,19 +798,19 @@ inline std::size_t FLineEdit::getCursorColumnPos() const
 }
 
 //----------------------------------------------------------------------
-inline FString FLineEdit::getPasswordText() const
+inline auto FLineEdit::getPasswordText() const -> FString
 {
   return {text.getLength(), UniChar::Bullet};  // •
 }
 
 //----------------------------------------------------------------------
-inline bool FLineEdit::isPasswordField() const
+inline auto FLineEdit::isPasswordField() const -> bool
 {
   return input_type == InputType::Password;
 }
 
 //----------------------------------------------------------------------
-inline FLineEdit::offsetPair FLineEdit::endPosToOffset (std::size_t pos)
+inline auto FLineEdit::endPosToOffset (std::size_t pos) -> offsetPair
 {
   std::size_t input_width = getWidth() - 2;
   std::size_t fullwidth_char_offset{0};
@@ -794,9 +872,9 @@ inline FLineEdit::offsetPair FLineEdit::endPosToOffset (std::size_t pos)
 }
 
 //----------------------------------------------------------------------
-std::size_t FLineEdit::clickPosToCursorPos (std::size_t pos)
+auto FLineEdit::clickPosToCursorPos (std::size_t pos) -> std::size_t
 {
-  std::size_t click_width{0};
+  std::size_t click_width{align_offset};
   std::size_t idx = text_offset;
   const std::size_t len = print_text.getLength();
   pos -= char_width_offset;
@@ -828,11 +906,11 @@ std::size_t FLineEdit::clickPosToCursorPos (std::size_t pos)
 //----------------------------------------------------------------------
 void FLineEdit::adjustTextOffset()
 {
-  const std::size_t input_width = getWidth() - 2;
-  const std::size_t len = print_text.getLength();
-  const std::size_t len_column = getColumnWidth (print_text);
-  std::size_t text_offset_column = getColumnWidth (print_text, text_offset);
-  const std::size_t cursor_pos_column = getColumnWidth (print_text, cursor_pos);
+  const auto input_width = getWidth() - 2;
+  const auto len = print_text.getLength();
+  const auto len_column = getColumnWidth (print_text);
+  auto text_offset_column = getColumnWidth (print_text, text_offset);
+  const auto cursor_pos_column = getColumnWidth (print_text, cursor_pos);
   std::size_t first_char_width{0};
   std::size_t cursor_char_width{1};
   char_width_offset = 0;
@@ -1005,12 +1083,7 @@ inline void FLineEdit::deletePreviousCharacter()
 //----------------------------------------------------------------------
 inline void FLineEdit::switchInsertMode()
 {
-  insert_mode = ! insert_mode;
-
-  if ( insert_mode )
-    FVTerm::getFOutput()->setCursor(CursorMode::Insert);     // Insert mode
-  else
-    FVTerm::getFOutput()->setCursor(CursorMode::Overwrite);  // Overtype mode
+  setOverwriteMode (insert_mode);
 }
 
 //----------------------------------------------------------------------
@@ -1020,7 +1093,7 @@ inline void FLineEdit::acceptInput()
 }
 
 //----------------------------------------------------------------------
-inline bool FLineEdit::keyInput (FKey key)
+inline auto FLineEdit::keyInput (FKey key) -> bool
 {
   if ( text.getLength() >= max_length )
   {
@@ -1059,14 +1132,12 @@ inline bool FLineEdit::keyInput (FKey key)
 }
 
 //----------------------------------------------------------------------
-inline wchar_t FLineEdit::characterFilter (const wchar_t c) const
+inline auto FLineEdit::characterFilter (const wchar_t c) const -> wchar_t
 {
   if ( input_filter.empty() )
     return c;
 
-  std::array<const wchar_t, 2> character{{c, L'\0'}};
-
-  if ( regex_match(character.data(), std::wregex(input_filter)) )
+  if ( std::regex_match(std::wstring(1, c), std::wregex(input_filter)) )
     return c;
 
   return L'\0';
