@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2014-2021 Markus Gans                                      *
+* Copyright 2014-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -58,9 +58,8 @@ FListBox::FListBox (FWidget* parent)
 //----------------------------------------------------------------------
 FListBox::~FListBox()  // destructor
 {
-  if ( source_container )
-    delete source_container;  // for lazy conversion
-
+  delete source_container;  // for lazy conversion
+  source_container = nullptr;
   delOwnTimers();
 }
 
@@ -157,7 +156,7 @@ void FListBox::hide()
 //----------------------------------------------------------------------
 void FListBox::insert (const FListBoxItem& listItem)
 {
-  const std::size_t column_width = getColumnWidth(listItem.text);
+  const auto column_width = getColumnWidth(listItem.text);
   const bool has_brackets(listItem.brackets != BracketType::None);
   recalculateHorizontalBar (column_width, has_brackets);
 
@@ -166,7 +165,7 @@ void FListBox::insert (const FListBoxItem& listItem)
   if ( current == 0 )
     current = 1;
 
-  std::size_t element_count = getCount();
+  auto element_count = getCount();
   recalculateVerticalBar (element_count);
   processChanged();
 }
@@ -181,7 +180,7 @@ void FListBox::remove (std::size_t item)
   const std::size_t element_count = getCount();
   max_line_width = 0;
 
-  for (auto&& listbox_item : itemlist)
+  for (const auto& listbox_item : itemlist)
   {
     const auto column_width = getColumnWidth(listbox_item.getText());
 
@@ -294,15 +293,12 @@ void FListBox::onMouseDown (FMouseEvent* ev)
   setWidgetFocus(this);
   const int yoffset_before = yoffset;
   const std::size_t current_before = current;
-  const int mouse_x = ev->getX();
-  const int mouse_y = ev->getY();
 
-  if ( mouse_x > 1 && mouse_x < int(getWidth())
-    && mouse_y > 1 && mouse_y < int(getHeight()) )
+  if ( isWithinListBounds(ev->getPos()) )
   {
     click_on_list = true;
     const std::size_t element_count = getCount();
-    current = std::size_t(yoffset + mouse_y - 1);
+    current = std::size_t(yoffset + ev->getY() - 1);
 
     if ( current > element_count )
       current = element_count;
@@ -334,21 +330,15 @@ void FListBox::onMouseUp (FMouseEvent* ev)
 {
   click_on_list = false;
 
-  if ( drag_scroll != DragScrollMode::None )
+  if ( isDragging(drag_scroll) )
     stopDragScroll();
 
-  if ( ev->getButton() == MouseButton::Left )
-  {
-    const int mouse_x = ev->getX();
-    const int mouse_y = ev->getY();
+  if ( ev->getButton() != MouseButton::Left )
+    return;
 
-    if ( mouse_x > 1 && mouse_x < int(getWidth())
-      && mouse_y > 1 && mouse_y < int(getHeight())
-      && ! isMultiSelection() )
-    {
-      processSelect();
-    }
-  }
+  if ( isWithinListBounds(ev->getPos())
+    && ! isMultiSelection() )
+    processSelect();
 }
 
 //----------------------------------------------------------------------
@@ -363,11 +353,9 @@ void FListBox::onMouseMove (FMouseEvent* ev)
 
   const std::size_t current_before = current;
   const int yoffset_before = yoffset;
-  const int mouse_x = ev->getX();
   const int mouse_y = ev->getY();
 
-  if ( mouse_x > 1 && mouse_x < int(getWidth())
-    && mouse_y > 1 && mouse_y < int(getHeight()) )
+  if ( isWithinListBounds(ev->getPos()) )
   {
     click_on_list = true;
     const std::size_t element_count = getCount();
@@ -413,20 +401,14 @@ void FListBox::onMouseMove (FMouseEvent* ev)
 //----------------------------------------------------------------------
 void FListBox::onMouseDoubleClick (FMouseEvent* ev)
 {
-  if ( ev->getButton() != MouseButton::Left )
+  if ( ev->getButton() != MouseButton::Left
+    || ! isWithinListBounds(ev->getPos()) )
     return;
 
-  const int mouse_x = ev->getX();
-  const int mouse_y = ev->getY();
+  if ( yoffset + ev->getY() - 1 > int(getCount()) )
+    return;
 
-  if ( mouse_x > 1 && mouse_x < int(getWidth())
-    && mouse_y > 1 && mouse_y < int(getHeight()) )
-  {
-    if ( yoffset + mouse_y - 1 > int(getCount()) )
-      return;
-
-    processClick();
-  }
+  processClick();
 }
 
 //----------------------------------------------------------------------
@@ -477,9 +459,9 @@ void FListBox::onWheel (FWheelEvent* ev)
   const std::size_t current_before = current;
   const int yoffset_before = yoffset;
   static constexpr int wheel_distance = 4;
-  const MouseWheel wheel = ev->getWheel();
+  const auto& wheel = ev->getWheel();
 
-  if ( drag_scroll != DragScrollMode::None )
+  if ( isDragging(drag_scroll) )
     stopDragScroll();
 
   if ( wheel == MouseWheel::Up )
@@ -505,25 +487,19 @@ void FListBox::onWheel (FWheelEvent* ev)
 }
 
 //----------------------------------------------------------------------
-void FListBox::onFocusIn (FFocusEvent*)
+void FListBox::onFocusIn (FFocusEvent* in_ev)
 {
-  if ( getStatusBar() )
-    getStatusBar()->drawMessage();
-
+  drawStatusBarMessage();
   inc_search.clear();
+  FWidget::onFocusIn(in_ev);
 }
 
 //----------------------------------------------------------------------
-void FListBox::onFocusOut (FFocusEvent*)
+void FListBox::onFocusOut (FFocusEvent* out_ev)
 {
-  if ( getStatusBar() )
-  {
-    getStatusBar()->clearMessage();
-    getStatusBar()->drawMessage();
-  }
-
   delOwnTimers();
   inc_search.clear();
+  FWidget::onFocusOut(out_ev);
 }
 
 
@@ -601,7 +577,7 @@ void FListBox::adjustSize()
 
 // private methods of FListBox
 //----------------------------------------------------------------------
-inline FString FListBox::getString (FListBoxItems::iterator iter)
+inline auto FListBox::getString (FListBoxItems::iterator iter) -> FString
 {
   return iter->getText();
 }
@@ -618,42 +594,49 @@ void FListBox::init()
 //----------------------------------------------------------------------
 inline void FListBox::mapKeyFunctions()
 {
-  key_map[FKey::Return]    = [this] { acceptSelection(); };
-  key_map[FKey::Enter]     = [this] { acceptSelection(); };
-  key_map[FKey::Up]        = [this] { onePosUp(); };
-  key_map[FKey::Down]      = [this] { onePosDown(); };
-  key_map[FKey::Left]      = [this] { scrollLeft(); };
-  key_map[FKey::Right]     = [this] { scrollRight(); };
-  key_map[FKey::Page_up]   = [this] { onePageUp(); };
-  key_map[FKey::Page_down] = [this] { onePageDown(); };
-  key_map[FKey::Home]      = [this] { firstPos(); };
-  key_map[FKey::End]       = [this] { lastPos(); };
-  key_map_result[FKey::Insert]        = [this] { return changeSelectionAndPosition(); };
-  key_map_result[FKey::Space]         = [this] { return spacebarProcessing(); };
-  key_map_result[FKey::Erase]         = [this] { return deletePreviousCharacter(); };
-  key_map_result[FKey::Backspace]     = [this] { return deletePreviousCharacter(); };
-  key_map_result[FKey::Escape]        = [this] { return skipIncrementalSearch(); };
-  key_map_result[FKey::Escape_mintty] = [this] { return skipIncrementalSearch(); };
+  key_map =
+  {
+    { FKey::Return    , [this] { acceptSelection(); } },
+    { FKey::Enter     , [this] { acceptSelection(); } },
+    { FKey::Up        , [this] { onePosUp(); } },
+    { FKey::Down      , [this] { onePosDown(); } },
+    { FKey::Left      , [this] { scrollLeft(); } },
+    { FKey::Right     , [this] { scrollRight(); } },
+    { FKey::Page_up   , [this] { onePageUp(); } },
+    { FKey::Page_down , [this] { onePageDown(); } },
+    { FKey::Home      , [this] { firstPos(); } },
+    { FKey::End       , [this] { lastPos(); } }
+  };
+
+  key_map_result =
+  {
+    { FKey::Insert        , [this] { return changeSelectionAndPosition(); } },
+    { FKey::Space         , [this] { return spacebarProcessing(); } },
+    { FKey::Erase         , [this] { return deletePreviousCharacter(); } },
+    { FKey::Backspace     , [this] { return deletePreviousCharacter(); } },
+    { FKey::Escape        , [this] { return skipIncrementalSearch(); } },
+    { FKey::Escape_mintty , [this] { return skipIncrementalSearch(); } }
+  };
 }
 
 //----------------------------------------------------------------------
 void FListBox::processKeyAction (FKeyEvent* ev)
 {
   const auto idx = ev->key();
-  const auto& entry = key_map[idx];
+  const auto& iter = key_map.find(idx);
 
-  if ( entry )
+  if ( iter != key_map.end() )
   {
-    entry();
+    iter->second();
     ev->accept();
   }
   else
   {
-    const auto& entry_result = key_map_result[idx];
+    const auto& iter_result = key_map_result.find(idx);
 
-    if ( entry_result )
+    if ( iter_result != key_map_result.end() )
     {
-      if ( entry_result() )
+      if ( iter_result->second() )
         ev->accept();
     }
     else if ( keyIncSearchInput(idx) )
@@ -694,18 +677,7 @@ void FListBox::draw()
 
   drawScrollbars();
   drawList();
-
-  if ( getFlags().focus && getStatusBar() )
-  {
-    const auto& msg = getStatusbarMessage();
-    const auto& curMsg = getStatusBar()->getMessage();
-
-    if ( curMsg != msg )
-    {
-      getStatusBar()->setMessage(msg);
-      getStatusBar()->drawMessage();
-    }
-  }
+  updateStatusbar (this);
 }
 
 //----------------------------------------------------------------------
@@ -822,9 +794,9 @@ inline void FListBox::drawListLine ( int y
   const std::size_t first = std::size_t(xoffset) + 1;
   const std::size_t max_width = getWidth() - nf_offset - 4;
   const FString element(getColumnSubString (getString(iter), first, max_width));
-  std::size_t column_width = getColumnWidth(element);
+  auto column_width = getColumnWidth(element);
 
-  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus )
+  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus.focus )
     print (UniChar::BlackRightPointingPointer);  // ►
   else
     print (' ');
@@ -835,14 +807,14 @@ inline void FListBox::drawListLine ( int y
 
   for (std::size_t i{0}; i < element.getLength(); i++)
   {
-    if ( serach_mark && i == inc_len && getFlags().focus  )
+    if ( serach_mark && i == inc_len && getFlags().focus.focus  )
       setColor ( wc->current_element_focus_fg
                , wc->current_element_focus_bg );
 
     print (element[i]);
   }
 
-  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine  && getFlags().focus )
+  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine  && getFlags().focus.focus )
   {
     print (UniChar::BlackLeftPointingPointer);  // ◄
     column_width++;
@@ -875,7 +847,7 @@ inline void FListBox::drawListBracketsLine ( int y
   const std::size_t inc_len = inc_search.getLength();
   const bool isCurrentLine( y + yoffset + 1 == int(current) );
 
-  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus )
+  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus.focus )
     print (UniChar::BlackRightPointingPointer);  // ►
   else
     print (' ');
@@ -889,7 +861,7 @@ inline void FListBox::drawListBracketsLine ( int y
   const auto first = std::size_t(xoffset);
   const std::size_t max_width = getWidth() - nf_offset - 4 - b;
   const FString element(getColumnSubString (getString(iter), first, max_width));
-  std::size_t column_width = getColumnWidth(element);
+  auto column_width = getColumnWidth(element);
   const std::size_t text_width = getColumnWidth(getString(iter));
   std::size_t i{0};
   const auto& wc = getColorTheme();
@@ -918,7 +890,7 @@ inline void FListBox::drawListBracketsLine ( int y
     column_width++;
   }
 
-  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus )
+  if ( FVTerm::getFOutput()->isMonochron() && isCurrentLine && getFlags().focus.focus )
   {
     print (UniChar::BlackLeftPointingPointer);   // ◄
     column_width++;
@@ -929,16 +901,9 @@ inline void FListBox::drawListBracketsLine ( int y
 }
 
 //----------------------------------------------------------------------
-inline void FListBox::setLineAttributes ( int y
-                                        , bool isLineSelected
-                                        , bool lineHasBrackets
-                                        , bool& serach_mark )
+inline void FListBox::setInitialLineAttributes (bool isLineSelected) const
 {
-  const bool isCurrentLine( y + yoffset + 1 == int(current) );
-  const std::size_t inc_len = inc_search.getLength();
-  const std::size_t inc_width = getColumnWidth(inc_search);
   const auto& wc = getColorTheme();
-  print() << FPoint{2, 2 + int(y)};
 
   if ( isLineSelected )
   {
@@ -954,58 +919,82 @@ inline void FListBox::setLineAttributes ( int y
     else
       setColor (wc->list_fg, wc->list_bg);
   }
+}
+
+//----------------------------------------------------------------------
+inline void FListBox::setCurrentLineAttributes ( int y
+                                               , bool isLineSelected
+                                               , bool lineHasBrackets
+                                               , bool& serach_mark )
+{
+  const auto& wc = getColorTheme();
+  const std::size_t inc_len = inc_search.getLength();
+  const std::size_t inc_width = getColumnWidth(inc_search);
+
+  if ( getFlags().focus.focus && FVTerm::getFOutput()->getMaxColor() < 16 )
+    setBold();
+
+  if ( isLineSelected )
+  {
+    if ( FVTerm::getFOutput()->isMonochron() )
+      setBold();
+    else if ( getFlags().focus.focus )
+      setColor ( wc->selected_current_element_focus_fg
+               , wc->selected_current_element_focus_bg );
+    else
+      setColor ( wc->selected_current_element_fg
+               , wc->selected_current_element_bg );
+
+    setCursorPos ({3, 2 + y});  // first character
+  }
+  else
+  {
+    if ( FVTerm::getFOutput()->isMonochron() )
+      unsetBold();
+
+    if ( getFlags().focus.focus )
+    {
+      setColor ( wc->current_element_focus_fg
+               , wc->current_element_focus_bg );
+      const int b = lineHasBrackets ? 1 : 0;
+
+      if ( inc_len > 0 )  // incremental search
+      {
+        serach_mark = true;
+        // Place the cursor on the last found character
+        setCursorPos ({2 + b + int(inc_width), 2 + y});
+      }
+      else  // only highlighted
+        setCursorPos ({3 + b, 2 + y});  // first character
+    }
+    else
+      setColor ( wc->current_element_fg
+               , wc->current_element_bg );
+  }
+
+  if ( FVTerm::getFOutput()->isMonochron() )
+    setReverse(false);
+}
+
+//----------------------------------------------------------------------
+inline void FListBox::setLineAttributes ( int y
+                                        , bool isLineSelected
+                                        , bool lineHasBrackets
+                                        , bool& serach_mark )
+{
+  const bool isCurrentLine( y + yoffset + 1 == int(current) );
+  print() << FPoint{2, 2 + y};
+  setInitialLineAttributes(isLineSelected);
 
   if ( isCurrentLine )
   {
-    if ( getFlags().focus && FVTerm::getFOutput()->getMaxColor() < 16 )
-      setBold();
-
-    if ( isLineSelected )
-    {
-      if ( FVTerm::getFOutput()->isMonochron() )
-        setBold();
-      else if ( getFlags().focus )
-        setColor ( wc->selected_current_element_focus_fg
-                 , wc->selected_current_element_focus_bg );
-      else
-        setColor ( wc->selected_current_element_fg
-                 , wc->selected_current_element_bg );
-
-      setCursorPos ({3, 2 + int(y)});  // first character
-    }
-    else
-    {
-      if ( FVTerm::getFOutput()->isMonochron() )
-        unsetBold();
-
-      if ( getFlags().focus )
-      {
-        setColor ( wc->current_element_focus_fg
-                 , wc->current_element_focus_bg );
-        const int b = lineHasBrackets ? 1 : 0;
-
-        if ( inc_len > 0 )  // incremental search
-        {
-          serach_mark = true;
-          // Place the cursor on the last found character
-          setCursorPos ({2 + b + int(inc_width), 2 + int(y)});
-        }
-        else  // only highlighted
-          setCursorPos ({3 + b, 2 + int(y)});  // first character
-      }
-      else
-        setColor ( wc->current_element_fg
-                 , wc->current_element_bg );
-    }
-
-    if ( FVTerm::getFOutput()->isMonochron() )
-      setReverse(false);
+    setCurrentLineAttributes(y, isLineSelected, lineHasBrackets, serach_mark);
   }
   else
   {
     if ( FVTerm::getFOutput()->isMonochron() )
       setReverse(true);
-    else if ( getFlags().focus && FVTerm::getFOutput()->getMaxColor() < 16 )
+    else if ( getFlags().focus.focus && FVTerm::getFOutput()->getMaxColor() < 16 )
       unsetBold();
   }
 }
@@ -1168,7 +1157,7 @@ void FListBox::wheelUp (int pagesize)
 //----------------------------------------------------------------------
 void FListBox::wheelDown (int pagesize)
 {
-  const std::size_t element_count = getCount();
+  const auto& element_count = getCount();
   auto yoffset_end = int(element_count - getClientHeight());
 
   if ( yoffset_end < 0 )
@@ -1192,7 +1181,7 @@ void FListBox::wheelDown (int pagesize)
 }
 
 //----------------------------------------------------------------------
-bool FListBox::dragScrollUp()
+auto FListBox::dragScrollUp() -> bool
 {
   if ( current == 1 )
   {
@@ -1205,9 +1194,9 @@ bool FListBox::dragScrollUp()
 }
 
 //----------------------------------------------------------------------
-bool FListBox::dragScrollDown()
+auto FListBox::dragScrollDown() -> bool
 {
-  const std::size_t element_count = getCount();
+  const auto& element_count = getCount();
 
   if ( current == element_count )
   {
@@ -1222,7 +1211,7 @@ bool FListBox::dragScrollDown()
 //----------------------------------------------------------------------
 void FListBox::dragUp (MouseButton mouse_button)
 {
-  if ( drag_scroll != DragScrollMode::None
+  if ( isDragging(drag_scroll)
     && scroll_distance < int(getClientHeight()) )
     scroll_distance++;
 
@@ -1247,7 +1236,7 @@ void FListBox::dragUp (MouseButton mouse_button)
 //----------------------------------------------------------------------
 void FListBox::dragDown (MouseButton mouse_button)
 {
-  if ( drag_scroll != DragScrollMode::None
+  if ( isDragging(drag_scroll)
     && scroll_distance < int(getClientHeight()) )
     scroll_distance++;
 
@@ -1301,7 +1290,7 @@ void FListBox::prevListItem (int distance)
 //----------------------------------------------------------------------
 void FListBox::nextListItem (int distance)
 {
-  const std::size_t element_count = getCount();
+  const auto& element_count = getCount();
   const auto yoffset_end = int(element_count - getClientHeight());
 
   if ( current == element_count )
@@ -1451,7 +1440,7 @@ inline void FListBox::firstPos()
 //----------------------------------------------------------------------
 inline void FListBox::lastPos()
 {
-  const std::size_t element_count = getCount();
+  const auto& element_count = getCount();
   const auto yoffset_end = int(element_count - getClientHeight());
   current = element_count;
 
@@ -1462,7 +1451,16 @@ inline void FListBox::lastPos()
 }
 
 //----------------------------------------------------------------------
-inline bool FListBox::skipIncrementalSearch()
+inline auto FListBox::isWithinListBounds (const FPoint& pos) const -> bool
+{
+  return pos.getX() > 1
+      && pos.getX() < int(getWidth())
+      && pos.getY() > 1
+      && pos.getY() < int(getHeight());
+}
+
+//----------------------------------------------------------------------
+inline auto FListBox::skipIncrementalSearch() -> bool
 {
   if ( inc_search.getLength() > 0 )
   {
@@ -1481,9 +1479,9 @@ inline void FListBox::acceptSelection()
 }
 
 //----------------------------------------------------------------------
-inline bool FListBox::spacebarProcessing()
+inline auto FListBox::spacebarProcessing() -> bool
 {
-  const std::size_t inc_len = inc_search.getLength();
+  const auto& inc_len = inc_search.getLength();
 
   if ( inc_len > 0 )  // Enter a spacebar for incremental search
   {
@@ -1526,11 +1524,11 @@ inline bool FListBox::spacebarProcessing()
 }
 
 //----------------------------------------------------------------------
-inline bool FListBox::changeSelectionAndPosition()
+inline auto FListBox::changeSelectionAndPosition() -> bool
 {
   if ( isMultiSelection() )
   {
-    std::size_t element_count = getCount();
+    const auto& element_count = getCount();
 
     if ( isSelected(current) )
       unselectItem(current);
@@ -1554,9 +1552,9 @@ inline bool FListBox::changeSelectionAndPosition()
 }
 
 //----------------------------------------------------------------------
-inline bool FListBox::deletePreviousCharacter()
+inline auto FListBox::deletePreviousCharacter() -> bool
 {
-  const std::size_t inc_len = inc_search.getLength();
+  const auto& inc_len = inc_search.getLength();
 
   if ( inc_len == 0 )
     return false;
@@ -1584,7 +1582,7 @@ inline bool FListBox::deletePreviousCharacter()
 }
 
 //----------------------------------------------------------------------
-inline bool FListBox::keyIncSearchInput (FKey key)
+inline auto FListBox::keyIncSearchInput (FKey key) -> bool
 {
   if ( key <= 0x20 || key > 0x10fff )
     return false;
@@ -1595,7 +1593,7 @@ inline bool FListBox::keyIncSearchInput (FKey key)
   else
     inc_search += wchar_t(key);
 
-  std::size_t inc_len = inc_search.getLength();
+  const auto& inc_len = inc_search.getLength();
   bool inc_found{false};
   auto iter = itemlist.begin();
 
@@ -1678,7 +1676,7 @@ void FListBox::lazyConvert(FListBoxItems::iterator iter, std::size_t y)
 //----------------------------------------------------------------------
 void FListBox::cb_vbarChange (const FWidget*)
 {
-  const FScrollbar::ScrollType scroll_type = vbar->getScrollType();
+  const auto scroll_type = vbar->getScrollType();
   static constexpr int wheel_distance = 4;
   const std::size_t current_before = current;
   int distance{1};
@@ -1713,7 +1711,7 @@ void FListBox::cb_vbarChange (const FWidget*)
       break;
 
     default:
-      break;
+      throw std::invalid_argument{"Invalid scroll type"};
   }
 
   if ( current_before != current )
@@ -1739,7 +1737,7 @@ void FListBox::cb_vbarChange (const FWidget*)
 //----------------------------------------------------------------------
 void FListBox::cb_hbarChange (const FWidget*)
 {
-  const FScrollbar::ScrollType scroll_type = hbar->getScrollType();
+  const auto scroll_type = hbar->getScrollType();
   static constexpr int wheel_distance = 4;
   static constexpr int padding_space = 2;  // 1 leading space + 1 trailing space
   int distance{1};
@@ -1774,7 +1772,7 @@ void FListBox::cb_hbarChange (const FWidget*)
       break;
 
     default:
-      break;
+      throw std::invalid_argument{"Invalid scroll type"};
   }
 
   if ( xoffset_before != xoffset )
