@@ -1,9 +1,9 @@
 /***********************************************************************
-* fobject-test.cpp - FPoint unit tests                                 *
+* fobject-test.cpp - FObject unit tests                                *
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2023 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -20,6 +20,9 @@
 * <http://www.gnu.org/licenses/>.                                      *
 ***********************************************************************/
 
+#include <chrono>
+#include <thread>
+
 #include <cppunit/BriefTestProgressListener.h>
 #include <cppunit/CompilerOutputter.h>
 #include <cppunit/extensions/HelperMacros.h>
@@ -38,23 +41,25 @@ namespace test
 //----------------------------------------------------------------------
 
 class FObject_protected : public finalcut::FObject
+                        , public finalcut::FTimer<finalcut::FObject>
 {
   public:
+    // Constructor
     FObject_protected() = default;
 
-    bool event (finalcut::FEvent* ev) override
+    auto event (finalcut::FEvent* ev) -> bool override
     {
       return finalcut::FObject::event(ev);
     }
 
-    FTimerList* getTimerList() const
+    auto getTimerList() const -> finalcut::FTimer<finalcut::FObject>::FTimerList*
     {
       return finalcut::FObject::getTimerList();
     }
 
-    uInt processEvent()
+    auto processEvent() -> uInt
     {
-      return processTimerEvent();
+      return finalcut::FObject::processTimerEvent();
     }
 
     void setWidgetProperty (bool property)
@@ -62,7 +67,7 @@ class FObject_protected : public finalcut::FObject
       finalcut::FObject::setWidgetProperty (property);
     }
 
-    bool isWidget()
+    auto isWidget() -> bool
     {
       return finalcut::FObject::isWidget();
     }
@@ -85,7 +90,7 @@ class FObject_timer : public finalcut::FObject
   public:
     FObject_timer() = default;
 
-    int getValue() const
+    auto getValue() const -> int
     {
       return value;
     }
@@ -109,7 +114,7 @@ class FObject_userEvent : public finalcut::FObject
   public:
     FObject_userEvent() = default;
 
-    int getValue() const
+    auto getValue() const -> int
     {
       return value;
     }
@@ -154,9 +159,6 @@ class FObjectTest : public CPPUNIT_NS::TestFixture
     void delTest();
     void elementAccessTest();
     void iteratorTest();
-    void timeTest();
-    void timerTest();
-    void performTimerActionTest();
     void userEventTest();
 
   private:
@@ -174,9 +176,6 @@ class FObjectTest : public CPPUNIT_NS::TestFixture
     CPPUNIT_TEST (delTest);
     CPPUNIT_TEST (elementAccessTest);
     CPPUNIT_TEST (iteratorTest);
-    CPPUNIT_TEST (timeTest);
-    CPPUNIT_TEST (timerTest);
-    CPPUNIT_TEST (performTimerActionTest);
     CPPUNIT_TEST (userEventTest);
 
     // End of test suite definition
@@ -221,7 +220,6 @@ void FObjectTest::noArgumentTest()
   CPPUNIT_ASSERT ( t.event(ev) );
   delete ev;
 
-  CPPUNIT_ASSERT ( ! finalcut::fc::emptyFString::get().isNull() );
   CPPUNIT_ASSERT ( finalcut::fc::emptyFString::get().isEmpty() );
 }
 
@@ -512,24 +510,11 @@ void FObjectTest::iteratorTest()
   CPPUNIT_ASSERT ( child2->getParent() == obj );
   CPPUNIT_ASSERT ( child3->getParent() == obj );
 
-  finalcut::FObject::const_iterator c_iter, c_last;
-  c_iter = obj->begin();
-  c_last = obj->end();
-  std::size_t i = 0;
-
-  while ( c_iter != c_last )
-  {
-    i++;
-    ++c_iter;
-  }
-
-  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
-  CPPUNIT_ASSERT ( i == 3 );
-
+  // iterator
   finalcut::FObject::iterator iter, last;
   iter = obj->begin();
   last = obj->end();
-  i = 0;
+  std::size_t i = 0;
 
   while ( iter != last )
   {
@@ -540,176 +525,82 @@ void FObjectTest::iteratorTest()
   CPPUNIT_ASSERT ( obj->numOfChildren() == i );
   CPPUNIT_ASSERT ( i == 3 );
 
-  delete obj;
-}
+  // const iterator 1
+  finalcut::FObject::const_iterator c_iter, c_last;
+  c_iter = obj->begin();
+  c_last = obj->end();
+  i = 0;
 
-//----------------------------------------------------------------------
-void FObjectTest::timeTest()
-{
-  TimeValue time1{};
-  uInt64 timeout = 750000;  // 750 ms
-  time1 = finalcut::FObject::getCurrentTime();
-  CPPUNIT_ASSERT ( ! finalcut::FObject::isTimeout (time1, timeout) );
-  sleep(1);
-  CPPUNIT_ASSERT ( finalcut::FObject::isTimeout (time1, timeout) );
-  time1 = TimeValue{}
-        + std::chrono::seconds(300)
-        + std::chrono::microseconds(2000000);
-  CPPUNIT_ASSERT ( finalcut::FObject::isTimeout (time1, timeout) );
-}
-
-//----------------------------------------------------------------------
-void FObjectTest::timerTest()
-{
-  using finalcut::operator +;
-  using finalcut::operator -;
-  using finalcut::operator +=;
-  using finalcut::operator <;
-
-  test::FObject_protected t1;
-  test::FObject_protected t2;
-  int id1, id2;
-  CPPUNIT_ASSERT ( t1.getTimerList()->empty() );
-  id1 = t1.addTimer(300);
-  CPPUNIT_ASSERT ( ! t1.getTimerList()->empty() );
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 1 );
-  id2 = t1.addTimer(900);
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 2 );
-  CPPUNIT_ASSERT ( &t1 != &t2 );
-  CPPUNIT_ASSERT ( id1 != id2 );
-  t1.delTimer (id1);
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 1 );
-  t1.delTimer (id2);
-  CPPUNIT_ASSERT ( t1.getTimerList()->empty() );
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 0 );
-
-  id1 = t1.addTimer(45);
-  id2 = t1.addTimer(95);
-  t1.delTimer (id2);
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 1 );
-  t1.delTimer (id1);
-  CPPUNIT_ASSERT ( t1.getTimerList()->empty() );
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 0 );
-
-  CPPUNIT_ASSERT ( ! t1.delTimer (id1) );  // id double delete
-  CPPUNIT_ASSERT ( ! t1.delAllTimers() );
-
-  t1.addTimer(250);
-  t1.addTimer(500);
-  t2.addTimer(750);
-  t2.addTimer(1000);
-  CPPUNIT_ASSERT_EQUAL ( t1.getTimerList(), t2.getTimerList() );
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 4 );
-  CPPUNIT_ASSERT ( t2.getTimerList()->size() == 4 );
-
-  t1.delOwnTimers();
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 2 );
-  CPPUNIT_ASSERT ( t2.getTimerList()->size() == 2 );
-
-  t1.addTimer(250);
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 3 );
-  CPPUNIT_ASSERT ( t2.getTimerList()->size() == 3 );
-
-  t2.delAllTimers();
-  CPPUNIT_ASSERT ( t1.getTimerList()->empty() );
-  CPPUNIT_ASSERT ( t2.getTimerList()->empty() );
-  CPPUNIT_ASSERT ( t1.getTimerList()->size() == 0 );
-  CPPUNIT_ASSERT ( t2.getTimerList()->size() == 0 );
-
-  auto tv1 = TimeValue{} + std::chrono::seconds(1321006271);
-  auto tv2 = TimeValue{} + std::chrono::seconds(27166271);
-  auto tv2_duration = std::chrono::duration_cast<std::chrono::seconds>(tv2.time_since_epoch());
-  auto tv_sum = tv1 + tv2_duration;
-  auto sec_sum = std::chrono::duration_cast<std::chrono::seconds>(tv_sum.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( sec_sum == 1348172542 );
-
-  auto tv_difference = tv1 - tv2_duration;
-  auto sec_difference = std::chrono::duration_cast<std::chrono::seconds>(tv_difference.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( sec_difference == 1293840000 );
-
-  tv_sum += tv2_duration;
-  sec_sum = std::chrono::duration_cast<std::chrono::seconds>(tv_sum.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( sec_sum == 1375338813 );
-
-  CPPUNIT_ASSERT ( tv2 < tv1 );
-  CPPUNIT_ASSERT ( ! (tv1 < tv2) );
-  CPPUNIT_ASSERT ( tv1 < tv_sum );
-  CPPUNIT_ASSERT ( ! (tv_sum < tv1) );
-  CPPUNIT_ASSERT ( tv2 < tv_sum );
-  CPPUNIT_ASSERT ( ! (tv_sum < tv2) );
-  CPPUNIT_ASSERT ( tv_difference < tv_sum );
-  CPPUNIT_ASSERT ( ! (tv_sum < tv_difference) );
-
-  tv1 += std::chrono::microseconds(600000);
-  tv2 += std::chrono::microseconds(600000);
-  tv2_duration = std::chrono::duration_cast<std::chrono::seconds>(tv2.time_since_epoch());
-  tv_sum = tv1 + tv2_duration;
-  auto s_sum = std::chrono::duration_cast<std::chrono::seconds>(tv_sum.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( s_sum == 1348172542 );
-  auto us_sum = ( std::chrono::duration_cast<std::chrono::microseconds>(tv_sum.time_since_epoch())
-                - std::chrono::seconds(1348172542) ).count();
-  CPPUNIT_ASSERT ( us_sum == 600000 );
-
-  auto tv1_sec = std::chrono::duration_cast<std::chrono::seconds>(tv1.time_since_epoch());
-  auto tv2_sec = std::chrono::duration_cast<std::chrono::seconds>(tv2.time_since_epoch());
-  tv1 = TimeValue{} + tv1_sec + std::chrono::microseconds(654321);
-  tv2 = TimeValue{} + tv2_sec + std::chrono::microseconds(123456);
-  auto tv2_duration_ms = std::chrono::duration_cast<std::chrono::microseconds>(tv2.time_since_epoch());
-  tv_difference = tv1 - tv2_duration_ms;
-  sec_difference = std::chrono::duration_cast<std::chrono::seconds>(tv_difference.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( sec_difference == 1293840000 );
-  auto usec_difference = ( std::chrono::duration_cast<std::chrono::microseconds>(tv_difference.time_since_epoch())
-                         - std::chrono::seconds(1293840000) ).count();
-  CPPUNIT_ASSERT ( usec_difference == 530865 );
-
-  tv2_sec = std::chrono::duration_cast<std::chrono::seconds>(tv2.time_since_epoch());
-  tv2 = TimeValue{} + tv2_sec + std::chrono::microseconds(999888);
-  auto tv2_duration2 = std::chrono::duration_cast<std::chrono::microseconds>(tv2.time_since_epoch());
-  tv_sum += tv2_duration2;
-  s_sum = std::chrono::duration_cast<std::chrono::seconds>(tv_sum.time_since_epoch()).count();
-  CPPUNIT_ASSERT ( s_sum == 1375338814 );
-  us_sum = ( std::chrono::duration_cast<std::chrono::microseconds>(tv_sum.time_since_epoch())
-           - std::chrono::seconds(1375338814) ).count();
-  CPPUNIT_ASSERT ( us_sum == 599888 );
-
-  CPPUNIT_ASSERT ( tv2 < tv1 );
-  CPPUNIT_ASSERT ( ! (tv1 < tv2) );
-  CPPUNIT_ASSERT ( tv_difference < tv_sum );
-  CPPUNIT_ASSERT ( ! (tv_sum < tv_difference) );
-
-  CPPUNIT_ASSERT ( ! t1.delTimer(0) );
-  CPPUNIT_ASSERT ( ! t1.delTimer(-1) );
-}
-
-//----------------------------------------------------------------------
-void FObjectTest::performTimerActionTest()
-{
-  test::FObject_protected t1;
-  uInt num_events = 0;
-  uInt loop = 0;
-  t1.addTimer(100);
-
-  while ( loop < 10 )
+  while ( c_iter != c_last )
   {
-    num_events += t1.processEvent();
-    // Wait 100 ms
-    const struct timespec ms[]{{0, 100000000L}};
-    nanosleep (ms, nullptr);
-    loop++;
+    i++;
+    ++c_iter;
   }
 
-  CPPUNIT_ASSERT ( loop == 10 );
-  CPPUNIT_ASSERT ( num_events == 9 );
-  CPPUNIT_ASSERT ( t1.count == 9 );
+  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
+  CPPUNIT_ASSERT ( i == 3 );
 
-  test::FObject_timer t2;
-  CPPUNIT_ASSERT ( t2.getValue() == 0 );
-  finalcut::FTimerEvent timer_ev (finalcut::Event::Timer, 1);
+  // const iterator 2
+  finalcut::FObject::const_iterator c_iter2, c_last2;
+  c_iter2 = obj->cbegin();
+  c_last2 = obj->cend();
+  i = 0;
 
-  for (auto x = 0; x < 10; x++)
-    finalcut::FApplication::sendEvent (&t2, &timer_ev);
+  while ( c_iter2 != c_last2 )
+  {
+    i++;
+    ++c_iter2;
+  }
 
-  CPPUNIT_ASSERT ( t2.getValue() == 10 );
+  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
+  CPPUNIT_ASSERT ( i == 3 );
+
+  // reverse iterator
+  finalcut::FObject::reverse_iterator r_iter, r_first;
+  r_iter = obj->rbegin();
+  r_first = obj->rend();
+  i = 0;
+
+  while ( r_iter != r_first )
+  {
+    i++;
+    ++r_iter;
+  }
+
+  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
+  CPPUNIT_ASSERT ( i == 3 );
+
+  // const reverse iterator 1
+  finalcut::FObject::const_reverse_iterator cr_iter, cr_first;
+  cr_iter = obj->rbegin();
+  cr_first = obj->rend();
+  i = 0;
+
+  while ( cr_iter != cr_first )
+  {
+    i++;
+    ++cr_iter;
+  }
+
+  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
+  CPPUNIT_ASSERT ( i == 3 );
+
+  // const reverse iterator 2
+  finalcut::FObject::const_reverse_iterator cr_iter2, cr_first2;
+  cr_iter2 = obj->crbegin();
+  cr_first2 = obj->crend();
+  i = 0;
+
+  while ( cr_iter2 != cr_first2 )
+  {
+    i++;
+    ++cr_iter2;
+  }
+
+  CPPUNIT_ASSERT ( obj->numOfChildren() == i );
+  CPPUNIT_ASSERT ( i == 3 );
+
+  delete obj;
 }
 
 //----------------------------------------------------------------------
