@@ -3,7 +3,7 @@
 *                                                                      *
 * This file is part of the FINAL CUT widget toolkit                    *
 *                                                                      *
-* Copyright 2018-2021 Markus Gans                                      *
+* Copyright 2018-2022 Markus Gans                                      *
 *                                                                      *
 * FINAL CUT is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License as       *
@@ -39,7 +39,8 @@
 // class FTermDetectionTest
 //----------------------------------------------------------------------
 
-class FTermDetectionTest : public CPPUNIT_NS::TestFixture, test::ConEmu
+class FTermDetectionTest : public CPPUNIT_NS::TestFixture
+                         , test::ConEmu
 {
   public:
     FTermDetectionTest() = default;
@@ -58,6 +59,7 @@ class FTermDetectionTest : public CPPUNIT_NS::TestFixture, test::ConEmu
     void teraTermTest();
     void cygwinTest();
     void minttyTest();
+    void sttermTest();
     void linuxTest();
     void freebsdTest();
     void netbsdTest();
@@ -88,6 +90,7 @@ class FTermDetectionTest : public CPPUNIT_NS::TestFixture, test::ConEmu
     CPPUNIT_TEST (teraTermTest);
     CPPUNIT_TEST (cygwinTest);
     CPPUNIT_TEST (minttyTest);
+    CPPUNIT_TEST (sttermTest);
     CPPUNIT_TEST (linuxTest);
     CPPUNIT_TEST (freebsdTest);
     CPPUNIT_TEST (netbsdTest);
@@ -149,6 +152,7 @@ void FTermDetectionTest::ansiTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -184,9 +188,13 @@ void FTermDetectionTest::ansiTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::ansi);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -228,6 +236,7 @@ void FTermDetectionTest::xtermTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -259,9 +268,13 @@ void FTermDetectionTest::xtermTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::xterm);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -302,6 +315,7 @@ void FTermDetectionTest::rxvtTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -330,9 +344,13 @@ void FTermDetectionTest::rxvtTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::rxvt);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -373,6 +391,7 @@ void FTermDetectionTest::urxvtTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -401,9 +420,13 @@ void FTermDetectionTest::urxvtTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::urxvt);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -444,6 +467,7 @@ void FTermDetectionTest::kdeKonsoleTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -472,9 +496,13 @@ void FTermDetectionTest::kdeKonsoleTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::kde_konsole);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -515,6 +543,7 @@ void FTermDetectionTest::gnomeTerminalTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -543,9 +572,13 @@ void FTermDetectionTest::gnomeTerminalTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::gnome_terminal);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -586,6 +619,7 @@ void FTermDetectionTest::newerVteTerminalTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -614,9 +648,13 @@ void FTermDetectionTest::newerVteTerminalTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::newer_vte_terminal);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -657,6 +695,7 @@ void FTermDetectionTest::puttyTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -686,9 +725,13 @@ void FTermDetectionTest::puttyTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::putty);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -731,6 +774,7 @@ void FTermDetectionTest::windowsTerminalTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -759,9 +803,13 @@ void FTermDetectionTest::windowsTerminalTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::win_terminal);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -802,6 +850,7 @@ void FTermDetectionTest::teraTermTest()
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -830,9 +879,13 @@ void FTermDetectionTest::teraTermTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::tera_term);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -873,6 +926,7 @@ void FTermDetectionTest::cygwinTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -909,9 +963,13 @@ void FTermDetectionTest::cygwinTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::cygwin);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -952,6 +1010,7 @@ void FTermDetectionTest::minttyTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -980,9 +1039,89 @@ void FTermDetectionTest::minttyTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::mintty);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
+  }
+}
+
+//----------------------------------------------------------------------
+void FTermDetectionTest::sttermTest()
+{
+  auto& data = finalcut::FTermData::getInstance();
+  finalcut::FTermDetection detect;
+  data.setTermType("st-256color");
+  detect.setTerminalDetection(true);
+
+  pid_t pid = forkConEmu();
+
+  if ( isConEmuChildProcess(pid) )
+  {
+    // (gdb) set follow-fork-mode child
+    setenv ("TERM", "st-256color", 1);
+    unsetenv("TERMCAP");
+    unsetenv("COLORTERM");
+    unsetenv("COLORFGBG");
+    unsetenv("VTE_VERSION");
+    unsetenv("XTERM_VERSION");
+    unsetenv("ROXTERM_ID");
+    unsetenv("KONSOLE_DBUS_SESSION");
+    unsetenv("KONSOLE_DCOP");
+    unsetenv("TMUX");
+    unsetenv("KITTY_WINDOW_ID");
+    detect.detect();
+
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::xterm) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::ansi) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::rxvt) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::urxvt) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::kde_konsole) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::gnome_terminal) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::putty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::win_terminal) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::stterm) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::openbsd_con) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::sun_con) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::screen) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tmux) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::kterm) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mlterm) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::kitty) );
+    CPPUNIT_ASSERT ( detect.canDisplay256Colors() );
+    CPPUNIT_ASSERT ( detect.hasTerminalDetection() );
+    CPPUNIT_ASSERT ( ! detect.hasSetCursorStyleSupport() );
+    CPPUNIT_ASSERT ( detect.getTermType() == "st-256color" );
+    CPPUNIT_ASSERT ( detect.getTermType_256color() == "" );
+    CPPUNIT_ASSERT ( detect.getTermType_Answerback() == "" );
+    CPPUNIT_ASSERT ( detect.getTermType_SecDA() == "" );
+    CPPUNIT_ASSERT ( detect.getAnswerbackString() == "" );
+    CPPUNIT_ASSERT ( detect.getSecDAString() == "" );
+
+    printConEmuDebug();
+    closeConEmuStdStreams();
+    exit(EXIT_SUCCESS);
+  }
+  else  // Parent
+  {
+    // Start the terminal emulation
+    startConEmuTerminal (ConEmu::console::stterm);
+    int wstatus;
+
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
+      std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1023,6 +1162,7 @@ void FTermDetectionTest::linuxTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1058,9 +1198,13 @@ void FTermDetectionTest::linuxTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::linux_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1102,6 +1246,7 @@ void FTermDetectionTest::freebsdTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1139,9 +1284,13 @@ void FTermDetectionTest::freebsdTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::freebsd_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1183,6 +1332,7 @@ void FTermDetectionTest::netbsdTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1218,9 +1368,13 @@ void FTermDetectionTest::netbsdTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::netbsd_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1262,6 +1416,7 @@ void FTermDetectionTest::openbsdTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1297,9 +1452,13 @@ void FTermDetectionTest::openbsdTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::openbsd_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1339,6 +1498,7 @@ void FTermDetectionTest::sunTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1374,9 +1534,13 @@ void FTermDetectionTest::sunTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::sun_con);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1417,6 +1581,7 @@ void FTermDetectionTest::screenTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1450,9 +1615,13 @@ void FTermDetectionTest::screenTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::screen);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1494,6 +1663,7 @@ void FTermDetectionTest::tmuxTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1570,6 +1740,7 @@ void FTermDetectionTest::ktermTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1608,9 +1779,13 @@ void FTermDetectionTest::ktermTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::kterm);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1652,6 +1827,7 @@ void FTermDetectionTest::mltermTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1686,9 +1862,13 @@ void FTermDetectionTest::mltermTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::mlterm);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1728,6 +1908,7 @@ void FTermDetectionTest::kittyTest()
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::tera_term) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::cygwin) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::mintty) );
+    CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::stterm) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::linux_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::freebsd_con) );
     CPPUNIT_ASSERT ( ! data.isTermType(finalcut::FTermType::netbsd_con) );
@@ -1760,9 +1941,13 @@ void FTermDetectionTest::kittyTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::kitty);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 }
 
@@ -1849,9 +2034,13 @@ void FTermDetectionTest::ttytypeTest()
   {
     // Start the terminal emulation
     startConEmuTerminal (ConEmu::console::ansi);
+    int wstatus;
 
-    if ( waitpid(pid, nullptr, WUNTRACED) != pid )
+    if ( waitpid(pid, &wstatus, WUNTRACED) != pid )
       std::cerr << "waitpid error" << std::endl;
+
+    if ( WIFEXITED(wstatus) )
+      CPPUNIT_ASSERT ( WEXITSTATUS(wstatus) == 0 );
   }
 
   unlink("new-root-dir/etc/ttytype");
